//AmpPhaseComboBox.java:  A combo box for choosing "Amplitude", "Phase"
//                        or "Amplitude/Phase".
//
//  5/13/2002 -- [ET]  Initial version.
//  3/25/2005 -- [ET]  Added "Show Stages" entries; added "is...()" methods.
//

package com.isti.jplotresp;

import javax.swing.JComboBox;

/**
 * Class AmpPhaseComboBox is a combo box for choosing "Amplitude", "Phase"
 * or "Amplitude/Phase".  Public static indices and strings are also
 * provided.
 */
public class AmpPhaseComboBox extends JComboBox
{
  /** String value for "Amplitude". */
  public static final String AMP_STR = "Amplitude";
  /** String value for "Phase". */
  public static final String PHASE_STR = "Phase";
  /** String value for "Amplitude/Phase". */
  public static final String AMP_PHASE_STR = "Amplitude/Phase";
  /** String value for "Amplitude". */
  public static final String SS_AMP_STR = "Show Stages Amplitude";
  /** String value for "Phase". */
  public static final String SS_PHASE_STR = "Show Stages Phase";
  /** String value for "Amplitude/Phase". */
  public static final String SS_AMPPHASE_STR = "Show Stages Amp/Phase";
  /** Index value for "Amplitude" (0). */
  public static final int AMP_IDX = 0;
  /** Index value for "Phase" (1). */
  public static final int PHASE_IDX = 1;
  /** Index value for "Amplitude/Phase" (2). */
  public static final int AMP_PHASE_IDX = 2;
  /** Index value for "Show Stages Amplitude" (3). */
  public static final int SS_AMP_IDX = 3;
  /** Index value for "Show Stages Phase" (4). */
  public static final int SS_PHASE_IDX = 4;
  /** Index value for "Show Stages Amp/Phase" (5). */
  public static final int SS_AMPPHASE_IDX = 5;
  /** Number of index values (6). */
  public static final int NUM_INDICES = 6;
  /** Default index value for combo box (AMP_PHASE_IDX). */
  public static final int DEFAULT_IDX = AMP_PHASE_IDX;
  /** Array of amplitude/phase strings for combo box. */
  public static final String [] AMP_PHASE_ARR = new String[]
                            { AMP_STR, PHASE_STR, AMP_PHASE_STR, SS_AMP_STR,
                                            SS_PHASE_STR, SS_AMPPHASE_STR };

  /**
   * Constructs a combo box for choosing "Amplitude", "Phase" or
   * "Amplitude/Phase".
   */
  public AmpPhaseComboBox()
  {
    super(AMP_PHASE_ARR);                   //create combo box
    setSelectedIndex(DEFAULT_IDX);          //set selected index
  }

  /**
   * Determines if the given selection index corresponds to an "Amplitude"
   * entry.
   * @param idx selection index to use.
   * @return true if the given selection index corresponds to an "Amplitude"
   * entry; false if not.
   */
  public static boolean isAmpSelIdx(int idx)
  {
    return (idx == AMP_IDX || idx == SS_AMP_IDX);
  }

  /**
   * Determines if the given selection index corresponds to a "Phase"
   * entry.
   * @param idx selection index to use.
   * @return true if the given selection index corresponds to a "Phase"
   * entry; false if not.
   */
  public static boolean isPhaseSelIdx(int idx)
  {
    return (idx == PHASE_IDX || idx == SS_PHASE_IDX);
  }

  /**
   * Determines if the given selection index corresponds to an
   * "Amplitude/Phase" entry.
   * @param idx selection index to use.
   * @return true if the given selection index corresponds to an
   * "Amplitude/Phase" entry; false if not.
   */
  public static boolean isAmpPhaseSelIdx(int idx)
  {
    return (idx == AMP_PHASE_IDX || idx == SS_AMPPHASE_IDX);
  }

  /**
   * Determines if the given selection index corresponds to any of the
   * "Show Stages..." entries.
   * @param idx selection index to use.
   * @return true if the given selection index corresponds to any of the
   * "Show Stages..." entries; false if not.
   */
  public static boolean isShowStagesSelIdx(int idx)
  {
    return (idx == SS_AMP_IDX || idx == SS_PHASE_IDX ||
                                                    idx == SS_AMPPHASE_IDX);
  }

  /**
   * Returns the amplitude/phase string asssociated with the given index.
   * If the index is out of range then the "Amplitude/Phase" string is
   * returned.
   * @param idx one of the "..._IDX" values.
   * @return One of the "..._STR" values.
   */
  public static String getStringForIndex(int idx)
  {
    if(idx < 0 || idx >= AMP_PHASE_ARR.length)
      idx = AMP_PHASE_IDX;
    return AMP_PHASE_ARR[idx];
  }
}
