//ChartPanelWithCopy.java:  Extends 'ChartPanel' to add a "Copy to Clipboard"
//                          option to the popup menu.
//
//  6/18/2007 -- [ET]  Code downloaded via SourceForge and modified.
//

package com.isti.jplotresp;

import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.DataFlavor;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.ChartPanel;

/**
 * Class ChartPanelWithCopy extends 'ChartPanel' to add a "Copy to Clipboard"
 * option to the popup menu.  Original code is from a JFreeChart patch
 * (1380913) posted to SourceForge; at:
 * https://sourceforge.net/tracker/index.php?func=detail&aid=1380913&group_id=15494&atid=315494
 * Original author:  Sergei Skarupo
 */
public class ChartPanelWithCopy extends ChartPanel implements ClipboardOwner
{
    /** Copy-to-clipboard action-command string. */
  public static final String COPY_COMMAND = "COPY_TO_CLIPBOARD";
    //array of 'DataFlavor' objects for 'Transferable' object:
  private static final DataFlavor [] transferDataFlavors =
                                                 { DataFlavor.imageFlavor };

  /**
   * Constructs a panel that displays the specified chart.
   * @param chart the chart.
   */
  public ChartPanelWithCopy(JFreeChart chart)
  {
    super(chart);
  }

  /**
   * Creates a popup menu for the chart panel.  This overridden version
   * add the "Copy to Clipboard" menu item.
   * @param properties include a menu item for the chart property editor.
   * @param save include a menu item for saving the chart.
   * @param print include a menu item for printing the chart.
   * @param zoom include menu items for zooming.
   * @return The popup menu.
   */
  protected JPopupMenu createPopupMenu(boolean properties, boolean save,
                                                boolean print, boolean zoom)
  {
    final JPopupMenu popupMenuObj = super.createPopupMenu(
                                                properties,save,print,zoom);
    popupMenuObj.addSeparator();                 //add separator to menu
                                                 //create menu item:
    final JMenuItem copyItem = new JMenuItem("Copy to Clipboard");
    copyItem.setActionCommand(COPY_COMMAND);     //set command for item
    copyItem.addActionListener(this);            //set 'this' as listener
    popupMenuObj.add(copyItem);                  //add item to menu
    return popupMenuObj;
  }

  /**
   * Handles action events generated by the popup menu.  This overridden
   * version adds handling for the "Copy to Clipboard" item.
   * @param event event object.
   */
  public void actionPerformed(ActionEvent event)
  {
    if(COPY_COMMAND.equals(event.getActionCommand()))
      copyChart();
    else
      super.actionPerformed(event);
  }

  public void copyChart()
  {
    //should this method and lostOwnership(Clipboard, Transferable)
    // be synchronized? [Sergei Skarupo]
    final Clipboard clipboardObj = getToolkit().getSystemClipboard();
    final Image imageObj =   //create image from chart panel
                     getChart().createBufferedImage(getWidth(),getHeight());
         //create 'Transferable' object for clipboard contents:
    final Transferable transferableObj = new Transferable()
        {
          public Object getTransferData(DataFlavor flavor)
          {
            return (isDataFlavorSupported(flavor)) ? imageObj : null;
          }
          public DataFlavor [] getTransferDataFlavors()
          {
            return transferDataFlavors;
          }
          public boolean isDataFlavorSupported(DataFlavor flavor)
          {
            return DataFlavor.imageFlavor.equals(flavor);
          }
        };
         //enter image contents into clipboard:
    clipboardObj.setContents(transferableObj,this);
  }

  /**
   * Notifies this object that it is no longer the clipboard owner.
   * This method implements the 'ClipboardOwner' interface.
   * @param clipboardObj the clipboard that is no longer owned.
   * @param contents contents which this owner had placed on the clipboard.
   */
  public void lostOwnership(Clipboard clipboardObj, Transferable contents)
  {
    //should this method and copyChart() be synchronized?  [Sergei Skarupo]
  }
}
