//FreqValuesSet.java:  Manages a set of 'FreqValueBlk' objects.
//
//  9/23/2005 -- [ET]
//

package com.isti.jplotresp;

import java.util.ArrayList;
import java.util.StringTokenizer;
import java.io.File;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import com.isti.jevalresp.AmpPhaseBlk;
import com.isti.util.ErrorMessageMgr;

public class FreqValuesSet extends ErrorMessageMgr
{
    /** Input file name. */
  protected final String fileNameStr;
    /** List of 'FreqValueBlk' objects. */
  protected ArrayList freqValuesListObj = null;

  /**
   * Creates a 'FreqValueBlk' objects set manager.
   * @param fileNameStr file name from which data will be read in (via
   * the 'readFileData()' method).
   */
  public FreqValuesSet(String fileNameStr)
  {
    this.fileNameStr = fileNameStr;
  }

  /**
   * Reads data in from the file named in the constructor.
   * @return true if successful; false if not (in which case an error
   * message may be fetched via the 'getErrorMessageString()' method).
   */
  public boolean readFileData()
  {
    final BufferedReader rdrObj;
    try
    {    //open file for input:
      rdrObj = new BufferedReader(new FileReader(fileNameStr));
    }
    catch(Exception ex)
    {    //error opening file; set error message
      setErrorMessageString("Error opening file:  " + ex);
      return false;
    }
    freqValuesListObj = new ArrayList();
    boolean retFlag = true;
    String lineStr;
    int lineCount = 0;
    StringTokenizer strTokObj;
    double freqVal,value1;
    boolean firstLineFlag = true, value2Flag = false;
    FreqValueBlk blkObj;
    try
    {
      while((lineStr=rdrObj.readLine()) != null)
      {  //for each line of data in file
        ++lineCount;                   //increment line count
        lineStr = lineStr.trim();      //trim any surrounding whitespace
        if(lineStr.length() > 0 && lineStr.charAt(0) != '#')
        {     //line contains data and does not start with comment character
          strTokObj = new StringTokenizer(lineStr);
          if(strTokObj.countTokens() < 2)
          {   //two few tokens on line; set error message
            setErrorMessageString("Less than two values on line " +
                                                                 lineCount);
            retFlag = false;      //setup error return flag
            break;                //exit input loop
          }
                   //parse first value as frequency:
          freqVal = Double.parseDouble(strTokObj.nextToken().trim());
                   //parse first data value:
          value1 = Double.parseDouble(strTokObj.nextToken().trim());
          if(firstLineFlag)
          {   //this is the first line processed
            firstLineFlag = false;          //clear indicator flag
            if(value2Flag = strTokObj.hasMoreTokens())
            {      //another token is available on the line
                        //parse second data value and create data block:
              blkObj = new FreqValueBlk(freqVal,value1,
                          Double.parseDouble(strTokObj.nextToken().trim()));
            }
            else   //no more tokens; create data block
              blkObj = new FreqValueBlk(freqVal,value1);
          }
          else
          {   //this is not the first line processed
            if(strTokObj.hasMoreTokens())
            {      //another token is available on the line
              if(!value2Flag)
              {    //previous lines only had 2 tokens; set error message
                setErrorMessageString("Too many values on line " +
                                                                 lineCount);
                retFlag = false;       //setup error return flag
                break;                 //exit input loop
              }
                        //parse second data value and create data block:
              blkObj = new FreqValueBlk(freqVal,value1,
                          Double.parseDouble(strTokObj.nextToken().trim()));
            }
            else
            {      //no more tokens
              if(value2Flag)
              {    //previous lines had 3 tokens; set error message
                setErrorMessageString("Too few values on line " +
                                                                 lineCount);
                retFlag = false;       //setup error return flag
                break;                 //exit input loop
              }
                        //create data block:
              blkObj = new FreqValueBlk(freqVal,value1);
            }
          }
          freqValuesListObj.add(blkObj);    //add data block to list
        }
      }
    }
    catch(NumberFormatException ex)
    {    //error parsing value; set error message
      setErrorMessageString("Illegal numeric value on line " + lineCount);
      retFlag = false;                 //setup error return flag
    }
    catch(Exception ex)
    {    //some kind of exception error; set error message
      setErrorMessageString("Error processing file (line=" + lineCount +
                                                                ":  " + ex);
      retFlag = false;                 //setup error return flag
    }
    try { rdrObj.close(); }
    catch(IOException ex) {}
    if(freqValuesListObj.size() <= 0)
    {    //no data blocks added to list; enter error msg (if no previous)
      enterErrorMessageString("No values found in file");
      retFlag = false;                 //setup error return flag
    }
    return retFlag;
  }

  /**
   * Determines if the frequency array of the given set object is equal
   * to the frequency array for this set object.
   * @param setObj the 'FreqValuesSet' object to use.
   * @return true if the frequency array of the given set object is equal
   * to the frequency array for this set object; false if not.
   */
  public boolean isFreqArrayEqual(FreqValuesSet setObj)
  {
    try
    {
      final int sizeVal;
      if(setObj != null && setObj.freqValuesListObj != null &&
                                                freqValuesListObj != null &&
                                (sizeVal=setObj.freqValuesListObj.size()) ==
                                                   freqValuesListObj.size())
      {  //objects are OK and number of entries are equal
        for(int i=0; i<sizeVal; ++i)
        {     //for each entry; check if frequency values are equal
          if(((FreqValueBlk)(setObj.freqValuesListObj.get(i))).freq !=
                            ((FreqValueBlk)(freqValuesListObj.get(i))).freq)
          {   //frequency values not equal
            return false;
          }
        }
        return true;
      }
    }
    catch(Exception ex)
    {    //some kind of exception error; just return false
    }
    return false;
  }

  /**
   * Returns the 'FreqValueBlk' for the given index in this set.
   * @param idx index value to use.
   * @return The 'FreqValueBlk' for the given index in this set, or null
   * if none.
   */
  public FreqValueBlk getBlkAt(int idx)
  {
    Object obj;
    return (freqValuesListObj != null && idx < freqValuesListObj.size() &&
                 (obj=freqValuesListObj.get(idx)) instanceof FreqValueBlk) ?
                                                   (FreqValueBlk)obj : null;
  }

  /**
   * Builds an array of frequency values using the frequency values from
   * this set.
   * @return A new array of 'double' values.
   */
  public double [] createFrequenciesArray()
  {
         //get size of 'freqValuesListObj' (or 0 if none):
    final int listSize = (freqValuesListObj != null) ?
                                               freqValuesListObj.size() : 0;
                                       //create 'AmpPhaseBlk' array:
    final double [] frequenciesArr = new double[listSize];
    FreqValueBlk blkObj;
    for(int i=0; i<listSize; ++i)
    {    //for each array entry to be filled
              //if block fetched OK then create and enter array entry:
      if((blkObj=getBlkAt(i)) != null)
        frequenciesArr[i] = blkObj.freq;
    }
    return frequenciesArr;
  }

  /**
   * Builds an array of 'AmpPhaseBlk' objects using the data values from
   * this set.  If the "value2" data values are not available then all of
   * returned "phase" values will be set to zero.
   * @return A new array of 'AmpPhaseBlk' objects.
   */
  public AmpPhaseBlk [] createAmpPhaseBlkArray()
  {
         //get size of 'freqValuesListObj' (or 0 if none):
    final int listSize = (freqValuesListObj != null) ?
                                               freqValuesListObj.size() : 0;
                                       //create 'AmpPhaseBlk' array:
    final AmpPhaseBlk [] ampPhaseBlkArr = new AmpPhaseBlk[listSize];
         //set flag if only "value1" values & file name starts with "PHASE":
    final boolean isPhaseFlag = !containsSecondValues() && isPhaseFileName();
    FreqValueBlk blkObj;
    for(int i=0; i<listSize; ++i)
    {    //for each 'AmpPhaseBlk' array entry to be filled
      if((blkObj=getBlkAt(i)) != null)
      {  //block fetched OK; create and enter array entry
                   //(if flag then only load phase values)
        ampPhaseBlkArr[i] = isPhaseFlag ?
                                        new AmpPhaseBlk(0.0,blkObj.value1) :
                               new AmpPhaseBlk(blkObj.value1,blkObj.value2);
      }
    }
    return ampPhaseBlkArr;
  }

  /**
   * Determines if the data held by this set contains "second" values
   * (both amp & phase).
   * @return true if the data held by this set contains "second" values
   * (both amp & phase); false if not.
   */
  public boolean containsSecondValues()
  {
    FreqValueBlk blkObj;     //return 'value2Flag' for first block in set:
    return ((blkObj=getBlkAt(0)) != null && blkObj.value2Flag);
  }

  /**
   * Determines if the frequency values held by this set are spaced
   * logarithmically.
   * @return true if the frequency values held by this set are spaced
   * logarithmically; false if not.
   */
  public boolean isFreqLogSpacing()
  {
         //frequency spacing is declared to be logarithmic if difference
         // between first two frequencies vs. difference between last two
         // frequencies is greater than 0.1% of average frequency value:
    final int listSize;
    if(freqValuesListObj != null && (listSize=freqValuesListObj.size()) > 3)
    {    //list of 'FreqValueBlk' objects is OK and is large enough
      Object obj;
              //get first frequency value:
      final double freq0Val = ((obj=freqValuesListObj.get(0)) instanceof
                             FreqValueBlk) ? ((FreqValueBlk)obj).freq : 0.0;
              //get second frequency value:
      final double freq1Val = ((obj=freqValuesListObj.get(1)) instanceof
                             FreqValueBlk) ? ((FreqValueBlk)obj).freq : 0.0;
              //get second-to-last frequency value:
      final double freqL1Val = ((obj=freqValuesListObj.get(listSize-2))
                  instanceof FreqValueBlk) ? ((FreqValueBlk)obj).freq : 0.0;
              //get last frequency value:
      final double freqLVal = ((obj=freqValuesListObj.get(listSize-1))
                  instanceof FreqValueBlk) ? ((FreqValueBlk)obj).freq : 0.0;
              //estimate average frequency value:
      final double avgVal = Math.abs((freq0Val + freq1Val + freqL1Val +
                                                             freqLVal) / 4);
              //calculate delta between first two frequencies:
      final double begDeltaVal = Math.abs(freq1Val-freq0Val);
              //calculate delta between last two frequencies:
      final double endDeltaVal = Math.abs(freqLVal-freqL1Val);
              //calculate difference between deltas:
      final double diffVal = Math.abs(endDeltaVal-begDeltaVal);
              //if avg > 0 && diff > 0.1% of avg then declare logarithmic:
      if(avgVal > 0.0 && diffVal / avgVal > 0.001)
        return true;
    }
    return false;
  }

  /**
   * Determines if the filename for this set starts with "AMP".
   * @return true if the filename for this set starts with "AMP";
   * false if not.
   */
  public boolean isAmpFileName()
  {
    return fileNameStr != null &&
                                fileNameStr.toUpperCase().startsWith("AMP");
  }

  /**
   * Determines if the filename for this set starts with "PHA".
   * @return true if the filename for this set starts with "PHA";
   * false if not.
   */
  public boolean isPhaseFileName()
  {
    return fileNameStr != null &&
                                fileNameStr.toUpperCase().startsWith("PHA");
  }

  /**
   * Returns the file name for this manager.
   * @return The file name for this manager.
   */
  public String getFileNameStr()
  {
    return fileNameStr;
  }

  /**
   * Returns the "ID" file name for this manager (the file name with any
   * leading path characters removed).
   * @return The "ID" file name for this manager.
   */
  public String getIDFNameStr()
  {
    return (fileNameStr != null) ? (new File(fileNameStr)).getName() : "???";
  }

  /**
   * Builds an array of 'AmpPhaseBlk' objects using the "value1"
   * value from the given sets.  If a set is omitted then all of
   * its corresponding values will be set to zero.
   * @param ampSetObj set to use for "amp" values, or null for none.
   * @param phaseSetObj set to use for "phase" values, or null for none.
   * @return A new array of 'AmpPhaseBlk' objects.
   */
  public static AmpPhaseBlk [] buildAmpPhaseBlkArray(
                         FreqValuesSet ampSetObj, FreqValuesSet phaseSetObj)
  {
         //get 'freqValuesListObj' for "amp" set (or null if none):
    final ArrayList ampBlksList = (ampSetObj != null) ?
                                         ampSetObj.freqValuesListObj : null;
         //get 'freqValuesListObj' for "phase" set (or null if none):
    final ArrayList phaseBlksList = (phaseSetObj != null) ?
                                       phaseSetObj.freqValuesListObj : null;
         //get size of 'freqValuesListObj' for "amp" set (or 0 if none):
    final int ampBlksSize = (ampBlksList != null) ? ampBlksList.size() : 0;
         //get size of 'freqValuesListObj' for "phase" set (or 0 if none):
    final int phaseBlksSize = (phaseBlksList != null) ?
                                                   phaseBlksList.size() : 0;
         //select larger size value for size of returned array:
    final int ampPhaseSize = (ampBlksSize >= phaseBlksSize) ? ampBlksSize :
                                                              phaseBlksSize;
                                       //create 'AmpPhaseBlk' array:
    final AmpPhaseBlk [] ampPhaseBlkArr = new AmpPhaseBlk[ampPhaseSize];
    FreqValueBlk blkObj;
    double ampVal,phaseVal;            //fill 'AmpPhaseBlk' array:
    for(int i=0; i<ampPhaseSize; ++i)
    {    //for each 'AmpPhaseBlk' array entry to be filled
      ampVal = (ampSetObj != null &&             //value from "amp" set
              (blkObj=ampSetObj.getBlkAt(i)) != null) ? blkObj.value1 : 0.0;
      phaseVal = (phaseSetObj != null &&         //value from "phase" set
            (blkObj=phaseSetObj.getBlkAt(i)) != null) ? blkObj.value1 : 0.0;
                                       //create and enter array entry:
      ampPhaseBlkArr[i] = new AmpPhaseBlk(ampVal,phaseVal);
    }
    return ampPhaseBlkArr;
  }


/*
  public static void main(String [] args)
  {
    if(args.length > 0)
    {
      final FreqValuesSet setObj = new FreqValuesSet(args[0]);
      if(setObj.readFileData())
      {
        final java.util.Iterator iterObj =
                                        setObj.freqValuesListObj.iterator();
        while(iterObj.hasNext())
        {
          System.out.println(iterObj.next());
        }
        System.out.println("Number of entries = " +
                                           setObj.freqValuesListObj.size());
      }
      else
        System.err.println(setObj.getErrorMessageString());
    }
    else
      System.err.println("Filename parameter missing");
  }
*/
}
