//JPlotResp.java:  Program launcher that detects if the "JEvalResp.jar"
//                 file is missing and shows an error message if so.
//
//   4/6/2005 -- [ET]
//

package com.isti.jplotresp;

import com.isti.util.gui.SetupLookAndFeel;
import com.isti.util.gui.IstiDialogPopup;

/**
 * Class JPlotResp is a program launcher that detects if the "JEvalResp.jar"
 * file is missing and shows an error message if so.
 */
public class JPlotResp
{
    //no instances of class allowed
  private JPlotResp()
  {
  }

  /**
   * Launches the program.  If the "JEvalResp.jar" file is missing then an
   * error message is sent to the console (stderr) and a popup dialog
   * is shown.
   * @param args array of command-line parameters.
   */
  public static void main(String [] args)
  {
    try
    {         //transfer execution of program:
      PlotRun.main(args);
    }
    catch(NoClassDefFoundError ex)
    {    //class not found
      final String exStr;
      if((exStr=ex.toString()) != null && exStr.indexOf("RunExt") >= 0)
      {  //missing class is "RunExt"
        final String errStr =          //setup error message
                        "Required program file \"JEvalResp.jar\" not found";
        System.err.println(errStr);    //send error message to console
        try
        {     //show error message in popup dialog:
          new SetupLookAndFeel();
          IstiDialogPopup.showMessageDialog(null,errStr,"Error");
        }
        catch(Throwable ex2)
        {     //error showing popup dialog; ignore it and exit
        }
      }
      else    //missing class is not "RunExt"
        ex.printStackTrace();     //send stack trace to console
      System.exit(1);        //exit program with error code
    }
  }
}
