//RespAboutPanel.java:  Implements the "About" dialog panel.
//
//  6/17/2002 -- [KF]  Taken out of RespEntryPanel.
//  6/27/2002 -- [KF]  Updates for JFreeChart 0.9.2.
//  7/18/2002 -- [ET]  Changed references for "LaunchBrowserButton" and
//                     "LaunchBrowserTextArea" to 'isti.util'.
//   5/8/2002 -- [ET]  Changed "com.jrefinery" to "org.jfree" for
//                     JFreeChart 0.9.8.
//  3/31/2005 -- [ET]  Added Java-version text to bottom of panel.
//  9/19/2011 -- [ET]  Modified to use updated IstiSmallLogo graphic.
//

package com.isti.jplotresp;

import java.awt.*;
import javax.swing.*;
import org.jfree.chart.JFreeChart;
import com.isti.util.LaunchBrowser;
import com.isti.util.gui.LaunchBrowserButton;
import com.isti.util.gui.LaunchBrowserTextArea;
import com.isti.util.gui.IstiSmallLogo;
import com.isti.jevalresp.Run;

/**
 * Class RespAboutPanel implements the "About" dialog panel.
 */
public class RespAboutPanel extends JPanel
{
  public static final String istiLinkText = "http://www.isti.com";
  public static final String irisLinkText = "http://www.iris.washington.edu";
  private final LaunchBrowser launchBrowserObj;

  /**
   * Creates the the "About" dialog panel.
   * @param launchBrowserObj 'LaunchBrowser' object used to display URL.
   */
  public RespAboutPanel(LaunchBrowser launchBrowserObj)
  {
    this.launchBrowserObj = launchBrowserObj;    //save launcher object
    try
    {
      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /**
   * Component initialization
   * @throws Exception
   */
  private void jbInit() throws Exception
  {
    final Color fgColor = Color.black;      //foreground (text) color
    final Color bgColor = Color.white;      //background color
    this.setBackground(bgColor);
    this.setForeground(fgColor);
    this.setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));

    //show JPlotResp revision information:
    final JLabel revLabelObj = new JLabel(PlotRun.REVISION_STR);
    revLabelObj.setFont(new Font("Serif",Font.BOLD,28));
    revLabelObj.setForeground(fgColor);     //make text be black
    //need to put label into panel to get background color:
    final JPanel revisionPanel = new JPanel(     //setup layout with no vgap
        new FlowLayout(FlowLayout.CENTER,5,0));
    revisionPanel.add(revLabelObj);
    revisionPanel.setBackground(bgColor);
    revisionPanel.setAlignmentX(0.5f);      //align panel in box
    //add empty border with extra space on top:
    revisionPanel.setBorder(BorderFactory.createEmptyBorder(10,0,0,0));

    //show JEvalResp revision information:
    final JLabel respLabelObj = new JLabel("using " + Run.REVISION_STR);
    respLabelObj.setFont(new Font("Serif",Font.BOLD,18));
    respLabelObj.setForeground(fgColor);   //make text be black
    //need to put label into panel to get background color:
    final JPanel respPanel = new JPanel(         //setup layout with no vgap
                                        new FlowLayout(FlowLayout.CENTER,5,0));
    respPanel.add(respLabelObj);
    respPanel.setBackground(bgColor);
    respPanel.setAlignmentX(0.5f);      //align panel in box
    //add empty border with extra space on bottom:
    respPanel.setBorder(BorderFactory.createEmptyBorder(0,0,20,0));

    //create ISTI launch browser "button":
    final JButton istiButton = LaunchBrowserButton.createButton(
        "Written by ISTI",
        new ImageIcon(IstiSmallLogo.dataArray),
        launchBrowserObj, istiLinkText);
    istiButton.setFont(new Font("Serif",Font.BOLD,26));
    istiButton.setBackground(bgColor);
    //put button into panel for centering:
    final JPanel istiPanel = new JPanel(     //setup layout with no vgap
                                        new FlowLayout(FlowLayout.CENTER,5,0));
    istiPanel.setBackground(bgColor);
    istiPanel.add(istiButton);
    istiPanel.setAlignmentX(0.5f);          //align panel in box

    //put in ISTI link label:
    final JLabel istiLinkLabelObj = new JLabel(istiLinkText);
    istiLinkLabelObj.setFont(new Font("Dialog",Font.PLAIN,14));
    istiLinkLabelObj.setForeground(fgColor);     //make text be black
    //need to put label into panel to get background color:
    final JPanel istiLinkPanel = new JPanel(     //setup layout with no vgap
        new FlowLayout(FlowLayout.CENTER,5,0));
    istiLinkPanel.add(istiLinkLabelObj);
    istiLinkPanel.setBackground(bgColor);
    istiLinkPanel.setAlignmentX(0.5f);      //align panel in box
    //add empty border with extra space on bottom:
    istiLinkPanel.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));

    //put in "for" label:
    final JLabel forLabelObj = new JLabel("for");
    forLabelObj.setFont(new Font("Dialog",Font.BOLD,14));
    forLabelObj.setForeground(fgColor);     //make text be black
    //need to put label into panel to get background color:
    final JPanel forPanel = new JPanel();
    forPanel.add(forLabelObj);
    forPanel.setBackground(bgColor);
    forPanel.setAlignmentX(0.5f);           //align panel in box

    //create IRIS launch browser "button":
    final JButton irisButton = LaunchBrowserButton.createButton(
        new ImageIcon(IRISSmall.dataArray),
        launchBrowserObj, irisLinkText);
    irisButton.setBackground(bgColor);
    //put button into panel for centering:
    final JPanel irisPanel = new JPanel(         //setup layout with no vgap
                                        new FlowLayout(FlowLayout.CENTER,5,0));
    irisPanel.setBackground(bgColor);
    irisPanel.add(irisButton);
    irisPanel.setAlignmentX(0.5f);          //align panel in box

    //put in "DMS" label:
    final JLabel dmsLabelObj = new JLabel("DMS");
    dmsLabelObj.setFont(new Font("Serif",Font.BOLD,16));
    dmsLabelObj.setForeground(fgColor);     //make text be black
    //need to put label into panel to get background color:
    final JPanel dmsPanel = new JPanel(     //setup layout with no vgap
                                       new FlowLayout(FlowLayout.CENTER,5,0));
    dmsPanel.add(dmsLabelObj);
    dmsPanel.setBackground(bgColor);
    dmsPanel.setAlignmentX(0.5f);      //align panel in box

    //put in graphing-engine message:
    final JPanel jFreeChartPanel = new JPanel();
    final JTextArea jFreeChartTextObj = new LaunchBrowserTextArea(
        "\n Graphing Engine:  " + JFreeChart.INFO.getName() + ", Version " + JFreeChart.INFO.getVersion() + " \n" +
        " (" + JFreeChart.INFO.getInfo() + ") \n",
        launchBrowserObj, JFreeChart.INFO.getInfo());
    jFreeChartTextObj.setBackground(bgColor);
    jFreeChartTextObj.setForeground(fgColor);     //make text be black
    jFreeChartTextObj.setFont(new Font("Dialog",Font.BOLD,14));
    jFreeChartPanel.add(jFreeChartTextObj);
    jFreeChartPanel.setBackground(bgColor);
    jFreeChartPanel.setAlignmentX(0.5f);           //align panel in box

    final JTextArea javaVerTextObj = new JTextArea("Using Java " +
        PlotRun.javaVersionString + " on " +
        launchBrowserObj.getOsNamePropString());
    javaVerTextObj.setEditable(false);      //text area not editable
    //need to put text area into panel to get background color:
    final JPanel javaVerPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
    javaVerPanel.add(javaVerTextObj);
    javaVerPanel.setBackground(bgColor);
    javaVerPanel.setAlignmentX(0.5f);           //align panel in box

    this.add(revisionPanel);       //add panels to box
    this.add(respPanel);
    this.add(istiPanel);
    this.add(istiLinkPanel);
    this.add(forPanel);
    this.add(irisPanel);
    this.add(dmsPanel);
    this.add(jFreeChartPanel);
    this.add(javaVerPanel);
    //add empty border with extra space to center in dialog:
    this.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
  }
}
