//RespConsolePanel.java:  Implements the console panel.
//
//  5/12/2002 -- [KF]  Initial version.
//  5/14/2002 -- [ET]  Added 'getContainsTextFlag()' method; renamed
//                     'exitProgram()' to 'exitCleanup()'.
//   6/4/2002 -- [KF]  Put back things for the GUI designer
//  3/31/2005 -- [ET]  Modified to explicitly setup monospace font.
//  7/15/2009 -- [KF]  Modified to support running
//                     JPlotResp from another application.
//

package com.isti.jplotresp;

import java.io.PrintStream;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.*;
import com.isti.util.gui.TextAreaOutputStream;

/**
 * Class RespConsolePanel implements the console panel.
 */
public class RespConsolePanel extends JPanel
{
  private final JTabbedPane tabbedPaneObj;
  private final String panelTitle = "Console";
  private GridBagLayout panel1GridBagLayout = new GridBagLayout();
  private JPanel panel1 = new JPanel();
  private GridBagLayout panelGridBagLayout = new GridBagLayout();
  private JScrollPane scrollPane1 = new JScrollPane();
  private JTextArea outputTextArea = new JTextArea();
  protected JButton closeButton = new JButton();

    //create TextAreaOutputStream and wrap PrintStream around it:
  private final PrintStream textStmObj =    //(enable auto-flush)
             new PrintStream(new TextAreaOutputStream(outputTextArea),true);

    //save current 'stdout' and 'stderr' stream objects:
  private final PrintStream sysOutSaveStream = System.out;
  private final PrintStream sysErrSaveStream = System.err;

  /**
   * Construct the console panel.
   * @param tabbedPaneObj is the JTabbedPane for the console panel.
   * @param setupStreamsFlag true to set up the streams, false otherwise.
   */
  public RespConsolePanel(JTabbedPane tabbedPaneObj, boolean setupStreamsFlag)
  {
    this.tabbedPaneObj = tabbedPaneObj;     //save tabbed-pane object
    jbInit();
    if (setupStreamsFlag)
    {
      System.setOut(textStmObj);
      System.setErr(textStmObj);
    }
  }

  /**
   * Get the console output.
   * @return the console output printstream.
   */
  public PrintStream getConsoleOut()
  {
    return textStmObj;
  }

  /** Add the panel to the tabbed pane */
  protected void addTabbedPane()
  {
    if(tabbedPaneObj != null)
    {
      // if the panel is not already on the tabbed pane
      if (tabbedPaneObj.indexOfComponent(this) == -1)
      {
        // add the panel to the tabbed pane
        tabbedPaneObj.addTab(panelTitle, this);
      }
      // select this panel
      tabbedPaneObj.setSelectedComponent(this);
    }
  }

  /** Remove the panel from the tabbed pane */
  protected void removeTabbedPane()
  {
    // remove the console panel
    if(tabbedPaneObj != null)
      tabbedPaneObj.remove(this);

    // clear the output text
    outputTextArea.setText("");
  }

  /**
   * @return true if the console is not empty.
   */
  protected boolean getContainsTextFlag()
  {
    final String str;
    return ((str=outputTextArea.getText()) != null && str.length() > 0);
  }

  /**
   * Component initialization
   */
  private void jbInit()
  {
    this.setLayout(panelGridBagLayout);
    // set the size for the GUI designer
    this.setSize(PlotRun.DEF_FRAME_WIDTH, PlotRun.DEF_FRAME_HEIGHT);

    panel1.setLayout(panel1GridBagLayout);
    this.add(panel1,new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.BOTH,
        new Insets(3, 5, 3, 5), 0, 0));

    outputTextArea.setEditable(false);      //make text not editable
                                            //setup monospace font:
    outputTextArea.setFont(new Font("Monospaced",Font.PLAIN,12));
    scrollPane1.getViewport().add(outputTextArea, null);
    panel1.add(scrollPane1,new GridBagConstraints(0, 0, 2, 1, 1.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.BOTH,
        new Insets(0, 0, 0, 0), 0, 0));

    closeButton.addActionListener(new ActionListener()
        {
          public void actionPerformed(ActionEvent e)
          {
            closeButton_actionPerformed(e);
          }
        });
    closeButton.setToolTipText("Close the console panel");
    closeButton.setText("Close");
    panel1.add(closeButton,new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(0, 0, 0, 0), 0, 0));
  }

  /**
   * Console Close button was pressed
   * @param e action event
   */
  private void closeButton_actionPerformed(ActionEvent e)
  {
    // remove the Console panel
    removeTabbedPane();
  }

  /**
   * Restores the original system streams.
   */
  protected void restoreStreams()
  {
    if (System.out == textStmObj)
      System.setOut(sysOutSaveStream);
    if (System.err == textStmObj)
      System.setErr(sysErrSaveStream);
  }

  /** Processing to be done when the program terminates */
  protected void close()
  {
    restoreStreams();
    textStmObj.close(); // close the TextAreaOutputStream
  }
}
