//RespEntryPanel.java:  Implements the entry panel.
//
//  5/12/2002 -- [KF]
//  5/15/2002 -- [ET]  Added column sizes to text fields to prevent resizing
//                     after layout (esp after data entered).
//   6/4/2002 -- [KF]  Re-ordered fields, added Filename/Server browse buttons,
//                     Make the initial cursor location be in the "Network" field,
//                     If accessing FISSURES server, append "  (from server)",
//                     Change the "Response Type" combo-box tool tip text.
//  6/11/2002 -- [KF]  Add parent to showDialog.
//  6/11/2002 -- [KF]  Show version of JFreeChart
//  6/17/2002 -- [KF]  Put about panel (RespAboutPanel) and
//                     NoBorderLinkButton into separate files,
//                     Used new class for showing dialogs (RespDialog),
//                     Fix the focus order of the verbose check box.
//  7/10/2002 -- [KF]  Content window have the initial keyboard focus
//                     (instead of the "Close" button) in the ViewHTMLPanel
//                     help window.
//  7/12/2002 -- [KF]  Added checkbox to enable multiple outputs,
//                     Moved the time fields into new class (RespTimePanel)
//                     with an etched border with title and add new set of
//                     time fields and use titles "Begin Time" and "End Time",
//                     After returning from browser set the focus on the
//                     associated text field.
//  7/17/2002 -- [ET]  Put "Filename:"/"Server:" fields into west/center/east
//                     panels for alignment (instead of fixed size on radio
//                     buttons).
//  7/19/2002 -- [ET]  Changed references for "ViewHTMLPanel" and
//                     "FileUtils" to 'isti.util'; changed "RespDialog"
//                     to "com.isti.util.gui.SimpleDialog"; moved file-
//                     chooser classes to isti.util.gui'.
//   8/6/2002 -- [ET]  Added tool-tips for Begin/End Time panels; changed
//                     access from 'protected' to "package" on objects
//                     directly-accessed from 'RespProcessingThread'; added
//                     "Header:" check-box.
//  3/27/2003 -- [KF]  Added output directory.
//  5/14/2003 -- [ET]  Modified to use new "outputDirectory" configuration
//                     property and have "Output Directory:" field filled
//                     at startup; added 'getTabKeyCompsArray()' method;
//                     added more tooltips.
//  5/28/2003 -- [KF]  Added persistence of entry-screen settings.
//  5/29/2003 -- [KF]  Do not use IstiFileChooser for Macintosh.
//  5/29/2003 -- [ET]  Modified to go back to defaults when persistence
//                     is disabled.
//  4/11/2005 -- [ET]  Modified to use 'IstiFileChooser.createFileChooser()'
//                     (work-around for Java 1.4.2 bug (4711700) that could
//                     result in 'NullPointerException' at startup);
//                     changed access level from "protected" to "public"
//                     on variables accessed from other modules; added
//                     "Use Delay" and "Show Input" checkboxes; removed
//                     hard-coded sizings on filename and server radio
//                     buttons.
// 10/31/2005 -- [ET]  Added components for List-blockette interpolation
//                     options; add maximum-value limit to number-of-
//                     frequencies text-edit field.
//  7/16/2009 -- [KF]  Modified to support running
//                     JPlotResp from another application.
//  5/26/2010 -- [ET]  Added "Unwrap Phase" and "Use Total Sensit" checkbox
//                     items; renamed "Use Delay" checkbox to "Use Est Delay"
//                     and updated tooltip.
//  1/30/2012 -- [ET]  Changed references to "FISSURES" in tooltips to
//                     "network".
//   4/2/2012 -- [ET]  Added "Multi-server plots" checkbox item.
// 10/29/2013 -- [ET]  Added 'b62_x' field.
//

package com.isti.jplotresp;

import java.io.File;
import java.io.PrintStream;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import com.isti.util.CfgPropItem;
import com.isti.util.IstiFileFilter;
import com.isti.util.LaunchBrowser;
import com.isti.util.gui.FilteredJTextField;
import com.isti.util.gui.SimpleDialog;
import com.isti.util.gui.IstiFileChooser;
import com.isti.util.gui.IstiDialogUtil;
import com.isti.util.gui.ViewHTMLPanelHandler;
import com.isti.jevalresp.Run;

/**
 * Class RespEntryPanel implements the entry panel.
 */
public class RespEntryPanel extends JPanel
{
  public static final String HELP_FNAME = "JPlotResp.html";
  private static final int FIELD1_SIZE = 4;      //text field size
  private static final int FIELD2_SIZE = 20;     //larger text field size
  //setup window frame width for file choosers:
  private static final int chooserFrameWidth = PlotRun.DEF_FRAME_WIDTH * 3/5;
  //setup window frame height for file choosers:
  private static final int chooserFrameHeight =
                                             PlotRun.DEF_FRAME_HEIGHT * 3/5;

  private final JTabbedPane tabbedPaneObj;
  private final PlotRun plotRunObj;
  private final JButton quitButton;
  private final ConfigManager cm;

  private final String panelTitle = "Entry";
  private final GridBagLayout panellLayout = new GridBagLayout();
  private final JScrollPane scrollPane1 = new JScrollPane();
  private final JPanel panel1 = new JPanel();
  private final GridBagLayout panel1Layout = new GridBagLayout();
  private final JPanel panel2 = new JPanel();
  private final GridBagLayout panel2Layout = new GridBagLayout();

  private final JLabel networkLabel = new JLabel();
  public final JTextField networkTextField = new JTextField(FIELD1_SIZE);
  private final JLabel stationLabel = new JLabel();
  public final JTextField stationTextField = new JTextField(FIELD1_SIZE);
  private final JLabel locationLabel = new JLabel();
  public final JTextField locationTextField = new JTextField(FIELD1_SIZE);
  private final JLabel channelLabel = new JLabel();
  public final JTextField channelTextField = new JTextField(FIELD1_SIZE);

  private final JLabel minFreqLabel = new JLabel();
  public final FilteredJTextField minFreqTextField = new FilteredJTextField(
        FIELD1_SIZE,FilteredJTextField.FLOAT_CHARS);
  private final JLabel maxFreqLabel = new JLabel();
  public final FilteredJTextField maxFreqTextField = new FilteredJTextField(
        FIELD1_SIZE,FilteredJTextField.FLOAT_CHARS);
  private final JLabel numFreqsLabel = new JLabel();
  public final FilteredJTextField numFreqsTextField = new FilteredJTextField(
        null,FIELD1_SIZE,FilteredJTextField.INTEGER_CHARS,true);

  private final JPanel panel3 = new JPanel();
  private final GridBagLayout panel3Layout = new GridBagLayout();
  public final RespTimePanel beginTimePanel =
      new RespTimePanel("Begin Time","Start of range or single date/time");
  public final RespTimePanel endTimePanel =
      new RespTimePanel("End Time","End of date/time range");
  public final JCheckBox multiOutCheckBox = new JCheckBox();
  private final JCheckBox rememberSettingsCheckBox = new JCheckBox();

  private final JPanel panel4 = new JPanel();
  private final GridBagLayout panel4Layout = new GridBagLayout();

  private final JPanel fnServMainPanel = new JPanel();
  private final BoxLayout fnServMainPanelLayout =
      new BoxLayout(fnServMainPanel,BoxLayout.X_AXIS);
  private final JPanel fnServWestPanel = new JPanel();
  private final BoxLayout fnServWestPanelLayout =
      new BoxLayout(fnServWestPanel,BoxLayout.Y_AXIS);
  private final JPanel fnServCenterPanel = new JPanel();
  private final BoxLayout fnServCenterPanelLayout =
      new BoxLayout(fnServCenterPanel,BoxLayout.Y_AXIS);
  private final JPanel fnServEastPanel = new JPanel();
  private final BoxLayout fnServEastPanelLayout =
      new BoxLayout(fnServEastPanel,BoxLayout.Y_AXIS);

//  private final Dimension radioButtonDimension = new Dimension(90, 25);
  private final ButtonGroup buttonGroup1 = new ButtonGroup();
  public final JRadioButton filenameRadioButton = new JRadioButton();
  public final JRadioButton serverRadioButton = new JRadioButton();
  public final JTextField filenameTextField = new JTextField(FIELD2_SIZE);
  public final JTextField serverTextField = new JTextField(FIELD2_SIZE);
  private final JButton filenameBrowseButton = new JButton();
  private final JButton serverBrowseButton = new JButton();
  private final JLabel startStageLabel = new JLabel();
  public final FilteredJTextField startStageTextField = new FilteredJTextField(
        FIELD1_SIZE, FilteredJTextField.INTEGER_CHARS);
  private final JLabel endStageLabel = new JLabel();
  public final FilteredJTextField endStageTextField = new FilteredJTextField(
        FIELD1_SIZE, FilteredJTextField.INTEGER_CHARS);
  private final JLabel unitsLabel = new JLabel();
  private final JLabel frequencySpacingLabel = new JLabel();
  private final JLabel responseTypeLabel = new JLabel();
  public final JCheckBox verboseCheckBox = new JCheckBox();
  public final JCheckBox headersCheckBox = new JCheckBox();
  public final JCheckBox useDelayCheckBox = new JCheckBox();
  public final JCheckBox showInputCheckBox = new JCheckBox();
  public final JCheckBox multiSvrCheckBox = new JCheckBox();
  public final JCheckBox unwrapPhaseCheckBox = new JCheckBox();
  public final JCheckBox totalSensitCheckBox = new JCheckBox();
  public final JCheckBox interpOutputCheckBox = new JCheckBox();
  public final JCheckBox interpInputCheckBox = new JCheckBox();
  private final JPanel interpPanel = new JPanel();
  private final GridBagLayout interpPanelLayout = new GridBagLayout();
  private final JLabel b62XValueLabel = new JLabel();
  public final FilteredJTextField b62XValueTextField = new FilteredJTextField(
                              null,5,FilteredJTextField.SIGNED_EFLOAT_CHARS);
  private final JLabel tensionLabel = new JLabel();
  public final FilteredJTextField tensionTextField = new FilteredJTextField(
                        null,5,FilteredJTextField.SIGNED_EFLOAT_CHARS,true);
  public final JComboBox unitsComboBox = new JComboBox();
  public final JComboBox frequencySpacingComboBox = new JComboBox();
  public final JComboBox responseTypeComboBox = new JComboBox();
  private final JPanel comboBoxesPanel = new JPanel();
  private final GridBagLayout comboBoxesPanelLayout = new GridBagLayout();
  private final BorderLayout outputDirectoryLayout = new BorderLayout(10,0);
  private final JPanel outputDirectoryPanel = new JPanel();
  private final JLabel outputDirectoryLabel = new JLabel();
  public final JTextField outputDirectoryTextField
      = new JTextField(FIELD2_SIZE);
  private final JButton outputDirectoryBrowseButton = new JButton();

  private final JPanel panel5 = new JPanel();
  private final GridBagLayout panel5Layout = new GridBagLayout();

//  private final JLabel plotLabel = new JLabel();
  public final PlotDisplayOptionPanel plotDisplayOptionPanel;
//  private final JButton plotAmpPhaseButton = new JButton();
//  private final JLabel plotSpaceEmptyLabel = new JLabel();

//  private final JPanel panel6 = new JPanel();
//  private final GridBagLayout panel6Layout = new GridBagLayout();
  private final JButton closeAllPlotsButton = new JButton();
  private final JButton setDefaultsAllPlotsButton = new JButton();

  private final JPanel panel7 = new JPanel();
  private final GridBagLayout panel7Layout = new GridBagLayout();

  private final JButton runEvalrespButton = new JButton();
  private final JButton plotButton = new JButton();
  private final JButton aboutButton = new JButton();
  private final JButton helpButton = new JButton();

  // handle for console panel object
  private RespConsolePanel consolePanel = null;

  private JFileChooser filenameChooser = null;
  private JFileChooser serverChooser = null;
  private JFileChooser outputDirectoryChooser = null;
//  private JFileChooser plotChooser = null;

    //utility class for launching web browser:
  private final LaunchBrowser launchBrowserObj = new LaunchBrowser();
    //dialog utility object for popup messages:
  protected final IstiDialogUtil dialogUtilObj;
    //view HTML panel handler:
  protected final ViewHTMLPanelHandler viewHTMLPanelHandler;

  /**
   * Construct the entry panel.
   * @param tabbedPaneObj is the JTabbedPane for the entry panel.
   * @param plotRunObj the calling 'PlotRun' object.
   * @param quitButton a "Quit" button to be displayed, or null for none.
   */
  public RespEntryPanel(JTabbedPane tabbedPaneObj,PlotRun plotRunObj,
                        JButton quitButton)
  {
    this.tabbedPaneObj = tabbedPaneObj;     //save tabbed-pane object
    this.plotRunObj = plotRunObj;           //save calling object
    this.quitButton = quitButton;           //save quit button

    dialogUtilObj = new IstiDialogUtil(plotRunObj.programFrameObj);
    viewHTMLPanelHandler = new ViewHTMLPanelHandler(dialogUtilObj,launchBrowserObj);

    plotDisplayOptionPanel = new PlotDisplayOptionPanel();

    cm = plotRunObj.cfgObj;
    plotDisplayOptionPanel.addToComponentsTable(cm);
    cm.addToComponentsTable(networkTextField, cm.networkProp, false);
    cm.addToComponentsTable(stationTextField, cm.stationProp, false);
    cm.addToComponentsTable(locationTextField, cm.locationProp, false);
    cm.addToComponentsTable(channelTextField, cm.channelProp, false);
    cm.addToComponentsTable(minFreqTextField, cm.minFreqProp, true);
    cm.addToComponentsTable(maxFreqTextField, cm.maxFreqProp, true);
    cm.addToComponentsTable(numFreqsTextField, cm.numFreqProp, true);
    cm.addToComponentsTable(beginTimePanel.yearTextField,
                         cm.beginTimeYearProp, false);
    cm.addToComponentsTable(beginTimePanel.julianDayTextField,
                         cm.beginTimeJulianDayProp, false);
    cm.addToComponentsTable(beginTimePanel.timeTextField,
                         cm.beginTimeTimeProp, false);
    cm.addToComponentsTable(endTimePanel.yearTextField,
                         cm.endTimeYearProp, false);
    cm.addToComponentsTable(endTimePanel.julianDayTextField,
                         cm.endTimeJulianDayProp, false);
    cm.addToComponentsTable(endTimePanel.timeTextField,
                         cm.endTimeTimeProp, false);
    cm.addToComponentsTable(multiOutCheckBox, cm.multiOutProp, true);
    cm.addToComponentsTable(rememberSettingsCheckBox,
                            cm.rememberSettingsProp, true);
    cm.addToComponentsTable(filenameRadioButton,
                            cm.filenameSelectedFlagProp, true);
    cm.addToComponentsTable(filenameTextField, cm.filenameProp, false);
    cm.addToComponentsTable(serverTextField, cm.serverFileProp, true);
    cm.addToComponentsTable(startStageTextField, cm.startStageProp, false);
    cm.addToComponentsTable(endStageTextField, cm.endStageProp, false);
    cm.addToComponentsTable(verboseCheckBox, cm.verboseProp, true);
    cm.addToComponentsTable(headersCheckBox, cm.headersProp, true);
    cm.addToComponentsTable(useDelayCheckBox, cm.useDelayProp, true);
    cm.addToComponentsTable(showInputCheckBox, cm.showInputProp, true);
    cm.addToComponentsTable(b62XValueTextField, cm.b62XValueProp, true);
    cm.addToComponentsTable(multiSvrCheckBox, cm.multiSvrFlagProp, true);
    cm.addToComponentsTable(unwrapPhaseCheckBox, cm.unwrapPhaseFlagProp,
                                                                      true);
    cm.addToComponentsTable(totalSensitCheckBox, cm.totalSensitFlagProp,
                                                                      true);
    cm.addToComponentsTable(interpOutputCheckBox, cm.interpOutputProp, true);
    cm.addToComponentsTable(interpInputCheckBox, cm.interpInputProp, true);
    cm.addToComponentsTable(tensionTextField, cm.interpTensionProp, true,
                                                                      true);
    cm.addToComponentsTable(unitsComboBox, cm.unitsProp, true);
    cm.addToComponentsTable(frequencySpacingComboBox,
                            cm.frequencySpacingProp, true);
    cm.addToComponentsTable(responseTypeComboBox, cm.responseTypeProp, true);
    cm.addToComponentsTable(outputDirectoryTextField,
                         cm.outputDirectoryProp, true);

    jbInit();

    cm.loadComponentConfigSettings();  //load component config settings

    String outDirStr = outputDirectoryTextField.getText();
    if (outDirStr.length() <= 0 || outDirStr.equals("."))
    {  //output directory name is empty or "."
            //use current directory, expanded out to full pathname
      outDirStr = new File(".").getAbsolutePath();
            //if ends with "." then remove it
      if(outDirStr.endsWith("."))
        outDirStr = outDirStr.substring(0,outDirStr.length()-1);
      final int len;
            //if not single char and ends with separator preceded by an
            // alphanumeric char (not "C:\") then remove separator:
      if((len=outDirStr.length()) > 1 && outDirStr.endsWith(File.separator)
              && Character.isUnicodeIdentifierPart(outDirStr.charAt(len-2)))
      {
        outDirStr = outDirStr.substring(0,len-1);
      }

      outputDirectoryTextField.setText(outDirStr);
    }

    //get file choosers built (to make their initial launches
    // go faster) but do it later in a separate thread:
    SwingUtilities.invokeLater(new Runnable()
    {
      public void run()
      {
        setupFileChoosers();
      }
    });
  }

  /**
   * Get the console output.
   * @return the console output printstream.
   */
  public PrintStream getConsoleOut()
  {
    if (consolePanel != null)
    {
      return consolePanel.getConsoleOut();
    }
    return System.err;
  }

  /** Add the panel to the tabbed pane */
  protected void addTabbedPane()
  {
    if(tabbedPaneObj != null)
    {
      // if the panel is not already on the tabbed pane
      if (tabbedPaneObj.indexOfComponent(this) == -1)
      {
        // add the panel to the tabbed pane
        tabbedPaneObj.addTab(panelTitle, this);
      }
      // select this panel
      selectEntryPanel();
    }
  }

  /**
   * Remove the Console Panel from the tabbed pane.
   */
  protected void removeConsolePanel()
  {
    if (consolePanel != null)
    {
      consolePanel.removeTabbedPane();
    }
  }

  /** Remove the panel from the tabbed pane */
  protected void removeTabbedPane()
  {
    // remove the entry panel
    if(tabbedPaneObj != null)
      tabbedPaneObj.remove(this);
  }

  /**
   * Select the entry panel on the tabbed pane.
   */
  protected void selectEntryPanel()
  {
    // select this panel
    tabbedPaneObj.setSelectedComponent(this);
  }

  /**
   * Component initialization
   */
  private void jbInit()
  {
    this.setLayout(panellLayout);
    // set the size for the GUI designer
    this.setSize(PlotRun.DEF_FRAME_WIDTH, PlotRun.DEF_FRAME_HEIGHT);
    panel1.setLayout(panel1Layout);
    scrollPane1.getViewport().add(panel1, null);

    this.add(scrollPane1, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.BOTH,
        new Insets(3, 5, 3, 5), 0, 0));

    panel2.setBorder(BorderFactory.createLoweredBevelBorder());
    panel2.setLayout(panel2Layout);
    panel1.add(panel2, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.BOTH,
        new Insets(0, 0, 0, 0), 0, 0));

    networkLabel.setForeground(Color.black);
    networkLabel.setHorizontalAlignment(SwingConstants.CENTER);
    networkLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    networkLabel.setText("Network: ");
    final String networkTipText = "Network name(s) to be matched";
    networkLabel.setToolTipText(networkTipText);
    panel2.add(networkLabel, new GridBagConstraints(0, 0, 1, 1, 0.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(5, 10, 0, 0), 0, 5));

    networkTextField.setToolTipText(networkTipText);
    panel2.add(networkTextField, new GridBagConstraints(1, 0, 1, 1, 1.0, 1.0,
        GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
        new Insets(5, 0, 0, 10), 3, 0));

    stationLabel.setForeground(Color.black);
    stationLabel.setHorizontalAlignment(SwingConstants.CENTER);
    stationLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    stationLabel.setText("Station: ");
    final String stationTextTip = "List of station names";
    stationLabel.setToolTipText(stationTextTip);
    panel2.add(stationLabel, new GridBagConstraints(2, 0, 1, 1, 0.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(5, 10, 0, 0), 0, 5));

    stationTextField.setToolTipText(stationTextTip);
    panel2.add(stationTextField, new GridBagConstraints(3, 0, 1, 1, 1.0, 1.0,
        GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
        new Insets(5, 0, 0, 10), 3, 0));

    locationLabel.setForeground(Color.black);
    locationLabel.setHorizontalAlignment(SwingConstants.CENTER);
    locationLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    locationLabel.setText("Location: ");
    final String locationTipText = "Location ID name(s) to be matched";
    locationLabel.setToolTipText(locationTipText);
    panel2.add(locationLabel, new GridBagConstraints(4, 0, 1, 1, 0.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(5, 10, 0, 0), 0, 5));

    locationTextField.setToolTipText(locationTipText);
    panel2.add(locationTextField, new GridBagConstraints(5, 0, 1, 1, 1.0, 1.0,
        GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
        new Insets(5, 0, 0, 10), 3, 0));

    channelLabel.setForeground(Color.black);
    channelLabel.setHorizontalAlignment(SwingConstants.CENTER);
    channelLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    channelLabel.setText("Channel: ");
    final String channelTextTip = "List of channel names";
    channelLabel.setToolTipText(channelTextTip);
    panel2.add(channelLabel, new GridBagConstraints(6, 0, 1, 1, 0.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(5, 10, 0, 0), 0, 5));

    channelTextField.setToolTipText(channelTextTip);
    panel2.add(channelTextField, new GridBagConstraints(7, 0, 1, 1, 1.0, 1.0,
        GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
        new Insets(5, 0, 0, 10), 3, 0));

    minFreqLabel.setForeground(Color.black);
    minFreqLabel.setHorizontalAlignment(SwingConstants.CENTER);
    minFreqLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    minFreqLabel.setText("Min Freq: ");
    final String minFreqTipText =
        "Minimum value for the range of frequency values";
    minFreqLabel.setToolTipText(minFreqTipText);
    panel2.add(minFreqLabel,    new GridBagConstraints(0, 1, 1, 1, 0.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(5, 10, 10, 0), 0, 5));

    minFreqTextField.setToolTipText(minFreqTipText);
    panel2.add(minFreqTextField, new GridBagConstraints(1, 1, 1, 1, 1.0, 1.0,
        GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
        new Insets(5, 0, 10, 10), 3, 0));

    maxFreqLabel.setForeground(Color.black);
    maxFreqLabel.setHorizontalAlignment(SwingConstants.CENTER);
    maxFreqLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    maxFreqLabel.setText("Max Freq: ");
    final String maxFreqTipText =
        "Maximum value for the range of frequency values";
    maxFreqLabel.setToolTipText(maxFreqTipText);
    panel2.add(maxFreqLabel, new GridBagConstraints(2, 1, 1, 1, 0.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(5, 10, 10, 0), 0, 5));

    maxFreqTextField.setToolTipText(maxFreqTipText);
    panel2.add(maxFreqTextField, new GridBagConstraints(3, 1, 1, 1, 1.0, 1.0,
        GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
        new Insets(5, 0, 10, 10), 3, 0));

    numFreqsLabel.setForeground(Color.black);
    numFreqsLabel.setHorizontalAlignment(SwingConstants.CENTER);
    numFreqsLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    numFreqsLabel.setText("Num Freqs: ");
    final String numFreqsTipText =
        "Number of frequencies in the range of frequency values";
    numFreqsLabel.setToolTipText(numFreqsTipText);
    panel2.add(numFreqsLabel, new GridBagConstraints(4, 1, 1, 1, 0.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(5, 10, 10, 0), 0, 5));

    numFreqsTextField.setToolTipText(numFreqsTipText);
    numFreqsTextField.setMaxValue(999999);
    panel2.add(numFreqsTextField, new GridBagConstraints(5, 1, 1, 1, 1.0, 1.0,
        GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
        new Insets(5, 0, 10, 10), 3, 0));

    panel3.setBorder(BorderFactory.createLoweredBevelBorder());
    panel3.setLayout(panel3Layout);
    panel1.add(panel3, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0
        ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));

    panel3.add(beginTimePanel, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
        ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));

    panel3.add(endTimePanel, new GridBagConstraints(1, 0, 1, 1, 1.0, 1.0
        ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));

    multiOutCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    multiOutCheckBox.setToolTipText("Enable multiple outputs with same 'net.sta.loc.cha' code");
    multiOutCheckBox.setText("Enable Multi-Date Outputs: ");
    multiOutCheckBox.setVerticalAlignment(SwingConstants.TOP);
    panel3.add(multiOutCheckBox,   new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0
        ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 10, 0, 0), 0, 0));

    rememberSettingsCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    rememberSettingsCheckBox.setToolTipText(
                      "Use same settings the next time the program is run");
    rememberSettingsCheckBox.setText("Remember Settings: ");
    rememberSettingsCheckBox.setVerticalAlignment(SwingConstants.TOP);
    panel3.add(rememberSettingsCheckBox,   new GridBagConstraints(1, 1, 1, 1, 1.0, 1.0
        ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 10, 0, 0), 0, 0));

    panel4.setBorder(BorderFactory.createLoweredBevelBorder());
    panel4.setLayout(panel4Layout);
    panel1.add(panel4, new GridBagConstraints(0, 2, 1, 1, 1.0, 1.0
        ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));

    fnServMainPanel.setLayout(fnServMainPanelLayout);
    panel4.add(fnServMainPanel, new GridBagConstraints(0, 1, 10, 1, 1.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
        new Insets(6, 6, 0, 10), 0, 0));

    fnServWestPanel.setLayout(fnServWestPanelLayout);
    fnServCenterPanel.setLayout(fnServCenterPanelLayout);
    fnServEastPanel.setLayout(fnServEastPanelLayout);

    filenameRadioButton.setForeground(Color.black);
//    filenameRadioButton.setMaximumSize(radioButtonDimension);
//    filenameRadioButton.setMinimumSize(radioButtonDimension);
//    filenameRadioButton.setPreferredSize(radioButtonDimension);
    filenameRadioButton.setText("Filenames:");
    filenameRadioButton.setToolTipText("Fetch responses from local files");
    buttonGroup1.add(filenameRadioButton);
    fnServWestPanel.add(filenameRadioButton);
    final int fnServVertStrutHeight = 15;
    fnServWestPanel.add(Box.createVerticalStrut(fnServVertStrutHeight));

    filenameTextField.addKeyListener(new java.awt.event.KeyAdapter()
        {
          public void keyPressed(KeyEvent e)
          {
            filenameTextField_keyPressed(e);
          }
        });
    filenameTextField.setToolTipText(
        "Name of files or directories to search for responses");
    fnServCenterPanel.add(filenameTextField);
    fnServCenterPanel.add(Box.createVerticalStrut(fnServVertStrutHeight));

    filenameBrowseButton.setEnabled(false);
    filenameBrowseButton.addActionListener(new ActionListener()
        {
          public void actionPerformed(ActionEvent e)
          {
            filenameBrowseButton_actionPerformed(e);
          }
        });
    filenameBrowseButton.setText("Browse");
    filenameBrowseButton.setToolTipText(
                               "Browse for response files and directories");
    fnServEastPanel.add(filenameBrowseButton);
    fnServEastPanel.add(Box.createVerticalStrut(fnServVertStrutHeight));

    serverRadioButton.setForeground(Color.black);
//    serverRadioButton.setMaximumSize(radioButtonDimension);
//    serverRadioButton.setMinimumSize(radioButtonDimension);
//    serverRadioButton.setPreferredSize(radioButtonDimension);
    serverRadioButton.setText("Server:");
    serverRadioButton.setToolTipText("Fetch responses from network server");
    buttonGroup1.add(serverRadioButton);
    fnServWestPanel.add(serverRadioButton);

    //if filename is not selected
    if (!filenameRadioButton.isSelected())
      serverRadioButton.setSelected(true);  //select server

    serverTextField.addKeyListener(new java.awt.event.KeyAdapter()
        {
          public void keyPressed(KeyEvent e)
          {
            serverTextField_keyPressed(e);
          }
        });
    serverTextField.setToolTipText(
        "Name of properties file for network server");
    fnServCenterPanel.add(serverTextField);

    serverBrowseButton.setEnabled(false);
    serverBrowseButton.addActionListener(new ActionListener()
        {
          public void actionPerformed(ActionEvent e)
          {
            serverBrowseButton_actionPerformed(e);
          }
        });
    serverBrowseButton.setText("Browse");
    serverBrowseButton.setToolTipText("Browse for server properties file");
    fnServEastPanel.add(serverBrowseButton);

    fnServMainPanel.add(fnServWestPanel);
    fnServMainPanel.add(Box.createHorizontalStrut(5));
    fnServMainPanel.add(fnServCenterPanel);
    fnServMainPanel.add(Box.createHorizontalStrut(10));
    fnServMainPanel.add(fnServEastPanel);

    final int sp1val = 5;         //vertical spacing value

    startStageLabel.setForeground(Color.black);
    startStageLabel.setHorizontalAlignment(SwingConstants.CENTER);
    startStageLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    startStageLabel.setText("Start Stage:  ");
    final String startStagTipText = "Stage or start of range of stages to be processed in the response";
    startStageLabel.setToolTipText(startStagTipText);
    panel4.add(startStageLabel,  new GridBagConstraints(0, 3, 1, 1, 0.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(sp1val, 10, 0, 0), 0, 0));
//        new Insets(0, 10, 0, 0), 0, 20));

    startStageTextField.setToolTipText(startStagTipText);
    panel4.add(startStageTextField, new GridBagConstraints(1, 3, 1, 1, 0.1, 1.0,
        GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
        new Insets(sp1val, 0, 0, 10), 3, 0));

    endStageLabel.setForeground(Color.black);
    endStageLabel.setHorizontalAlignment(SwingConstants.CENTER);
    endStageLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    endStageLabel.setText("End Stage:  ");
    final String endStageTipText = "End of range of stages to be processed in the response";
    endStageLabel.setToolTipText(endStageTipText);
    panel4.add(endStageLabel,  new GridBagConstraints(2, 3, 1, 1, 0.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(sp1val, 10, 0, 0), 0, 0));
//        new Insets(0, 10, 0, 0), 0, 20));

    endStageTextField.setToolTipText(endStageTipText);
    panel4.add(endStageTextField, new GridBagConstraints(3, 3, 1, 1, 0.1, 1.0,
        GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
        new Insets(sp1val, 0, 0, 10), 3, 0));

    verboseCheckBox.setForeground(Color.black);
    verboseCheckBox.setHorizontalAlignment(SwingConstants.CENTER);
    verboseCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    verboseCheckBox.setText("Verbose: ");
    verboseCheckBox.setToolTipText(
        "Specifies that verbose messages should be shown");
    panel4.add(verboseCheckBox, new GridBagConstraints(4, 3, 1, 1, 0.1, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(sp1val, 0, 0, 0), 0, 0));

    headersCheckBox.setForeground(Color.black);
    headersCheckBox.setHorizontalAlignment(SwingConstants.CENTER);
    headersCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    headersCheckBox.setText("Headers: ");
    headersCheckBox.setToolTipText(
        "Enables header information in output files");
    panel4.add(headersCheckBox, new GridBagConstraints(5, 3, 1, 1, 0.1, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(sp1val, 0, 0, 0), 0, 0));

    useDelayCheckBox.setForeground(Color.black);
    useDelayCheckBox.setHorizontalAlignment(SwingConstants.CENTER);
    useDelayCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    useDelayCheckBox.setText("Use Est Delay: ");
    useDelayCheckBox.setToolTipText(
        "Use estimated delay in FIR_ASYM calculation");
    panel4.add(useDelayCheckBox, new GridBagConstraints(6, 3, 1, 1, 0.1, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(sp1val, 0, 0, 0), 0, 0));

    showInputCheckBox.setForeground(Color.black);
    showInputCheckBox.setHorizontalAlignment(SwingConstants.CENTER);
    showInputCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    showInputCheckBox.setText("Show Input: ");
    showInputCheckBox.setToolTipText(
        "Show input RESP text (sent to console)");
    panel4.add(showInputCheckBox, new GridBagConstraints(7, 3, 1, 1, 0.1, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(sp1val, 0, 0, 0), 0, 0));

    b62XValueLabel.setForeground(Color.black);
    b62XValueLabel.setHorizontalAlignment(SwingConstants.CENTER);
    b62XValueLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    b62XValueLabel.setText("b62_x:  ");
    final String b62XValueTipText = "Sample value for polynomial blockette";
    b62XValueLabel.setToolTipText(b62XValueTipText);
    panel4.add(b62XValueLabel, new GridBagConstraints(8, 3, 1, 1, 0.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(sp1val, 10, 0, 0), 0, 0));

    b62XValueTextField.setToolTipText(b62XValueTipText);
    b62XValueTextField.setMaxValue(9.9999e99);
    panel4.add(b62XValueTextField, new GridBagConstraints(9, 3, 1, 1, 0.1, 1.0,
        GridBagConstraints.WEST, GridBagConstraints.NONE,
        new Insets(sp1val, 0, 0, 0), 0, 0));

    interpPanel.setLayout(interpPanelLayout);

    multiSvrCheckBox.setForeground(Color.black);
    multiSvrCheckBox.setHorizontalAlignment(SwingConstants.CENTER);
    multiSvrCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    multiSvrCheckBox.setText("Multi-server plots: ");
    multiSvrCheckBox.setToolTipText(
                           "Fetch from all specified web-services servers");
    interpPanel.add(multiSvrCheckBox, new GridBagConstraints(0, 1, 1, 1, 3.0, 1.0,
        GridBagConstraints.WEST, GridBagConstraints.NONE,
        new Insets(0, 0, 0, 0), 0, 0));

    unwrapPhaseCheckBox.setForeground(Color.black);
    unwrapPhaseCheckBox.setHorizontalAlignment(SwingConstants.CENTER);
    unwrapPhaseCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    unwrapPhaseCheckBox.setText("Unwrap Phase: ");
    unwrapPhaseCheckBox.setToolTipText("Unwrap phase output values");
    interpPanel.add(unwrapPhaseCheckBox, new GridBagConstraints(1, 1, 1, 1, 1.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(0, 0, 0, 0), 0, 0));

    totalSensitCheckBox.setForeground(Color.black);
    totalSensitCheckBox.setHorizontalAlignment(SwingConstants.CENTER);
    totalSensitCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    totalSensitCheckBox.setText("Use Total Sensit: ");
    totalSensitCheckBox.setToolTipText("Use stage 0 (total) sensitivity");
    interpPanel.add(totalSensitCheckBox, new GridBagConstraints(2, 1, 1, 1, 1.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(0, 0, 0, 0), 0, 0));

    interpOutputCheckBox.setForeground(Color.black);
    interpOutputCheckBox.setHorizontalAlignment(SwingConstants.CENTER);
    interpOutputCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    interpOutputCheckBox.setText("Interpolate Output: ");
    interpOutputCheckBox.setToolTipText("Interpolate List blockette output");
    interpPanel.add(interpOutputCheckBox, new GridBagConstraints(3, 1, 1, 1, 1.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(0, 0, 0, 0), 0, 0));

    interpInputCheckBox.setForeground(Color.black);
    interpInputCheckBox.setHorizontalAlignment(SwingConstants.CENTER);
    interpInputCheckBox.setHorizontalTextPosition(SwingConstants.LEADING);
    interpInputCheckBox.setText("Interpolate Input: ");
    interpInputCheckBox.setToolTipText("Interpolate List blockette input");
    interpPanel.add(interpInputCheckBox, new GridBagConstraints(4, 1, 1, 1, 1.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE,
        new Insets(0, 0, 0, 0), 0, 0));

    tensionLabel.setForeground(Color.black);
    tensionLabel.setHorizontalAlignment(SwingConstants.CENTER);
    tensionLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    tensionLabel.setText("Tension:  ");
    final String tensionTipText = "Tension for List blockette interpolation";
    tensionLabel.setToolTipText(tensionTipText);
    interpPanel.add(tensionLabel,  new GridBagConstraints(5, 1, 1, 1, 1.0, 1.0,
        GridBagConstraints.EAST, GridBagConstraints.NONE,
        new Insets(0, 0, 0, 0), 0, 0));

    tensionTextField.setToolTipText(tensionTipText);
    tensionTextField.setMaxValue(9.9999e99);
    interpPanel.add(tensionTextField,  new GridBagConstraints(6, 1, 1, 1, 1.0, 1.0,
        GridBagConstraints.WEST, GridBagConstraints.NONE,
        new Insets(0, 0, 0, 0), 3, 0));

    panel4.add(interpPanel, new GridBagConstraints(0, 4, 10, 1, 1.0, 1.0,
                          GridBagConstraints.NORTH, GridBagConstraints.BOTH,
                                             new Insets(0, 6, 0, 0), 0, 0));

    comboBoxesPanel.setLayout(comboBoxesPanelLayout);
    unitsLabel.setForeground(Color.black);
    unitsLabel.setHorizontalAlignment(SwingConstants.CENTER);
    unitsLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    unitsLabel.setText("Output Units: ");
    final String unitsTipText = "Output units conversion parameter";
    unitsLabel.setToolTipText(unitsTipText);
    comboBoxesPanel.add(unitsLabel, new GridBagConstraints(
                             0, 0, 1, 1, 1.0, 0.0, GridBagConstraints.SOUTH,
                    GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));

    unitsComboBoxAddItems();
    unitsComboBox.setToolTipText(unitsTipText);
    comboBoxesPanel.add(unitsComboBox, new GridBagConstraints(
                             0, 1, 1, 1, 1.0, 0.5, GridBagConstraints.NORTH,
                    GridBagConstraints.NONE, new Insets(5, 0, 5, 0), 6, 0));

    frequencySpacingLabel.setForeground(Color.black);
    frequencySpacingLabel.setHorizontalAlignment(SwingConstants.CENTER);
    frequencySpacingLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    frequencySpacingLabel.setText("Frequency Spacing: ");
    final String frequencySpacingTipText = "Type of frequency spacing";
    frequencySpacingLabel.setToolTipText(frequencySpacingTipText);
    comboBoxesPanel.add(frequencySpacingLabel, new GridBagConstraints(
                             1, 0, 1, 1, 1.0, 0.0, GridBagConstraints.SOUTH,
                    GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));

    frequencySpacingComboBoxAddItems();
    frequencySpacingComboBox.setToolTipText(frequencySpacingTipText);
    comboBoxesPanel.add(frequencySpacingComboBox, new GridBagConstraints(
                             1, 1, 1, 1, 1.0, 0.5, GridBagConstraints.NORTH,
                    GridBagConstraints.NONE, new Insets(5, 0, 5, 0), 6, 0));

    responseTypeLabel.setForeground(Color.black);
    responseTypeLabel.setHorizontalAlignment(SwingConstants.CENTER);
    responseTypeLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    responseTypeLabel.setText("Output-File Type: ");
    final String responseTypeTipText =
        "Selects the type of output file(s) to be generated";
    responseTypeLabel.setToolTipText(responseTypeTipText);
    comboBoxesPanel.add(responseTypeLabel, new GridBagConstraints(
                             2, 0, 1, 1, 1.0, 0.0, GridBagConstraints.SOUTH,
                    GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));

    responseTypeComboBoxAddItems();
    responseTypeComboBox.setToolTipText(responseTypeTipText);
    comboBoxesPanel.add(responseTypeComboBox, new GridBagConstraints(
                             2, 1, 1, 1, 1.0, 0.5, GridBagConstraints.NORTH,
                    GridBagConstraints.NONE, new Insets(5, 0, 5, 0), 6, 0));

    panel4.add(comboBoxesPanel, new GridBagConstraints(0, 5, 10, 1, 1.0, 1.0,
                          GridBagConstraints.NORTH, GridBagConstraints.BOTH,
                                             new Insets(3, 0, 0, 0), 0, 0));

    outputDirectoryLabel.setText("Output Directory:");
    outputDirectoryPanel.setLayout(outputDirectoryLayout);
    outputDirectoryPanel.add(outputDirectoryLabel,  BorderLayout.WEST);
    final String outputDirectoryTipStr =
                                "Destination directory for generated files";
    outputDirectoryPanel.setToolTipText(outputDirectoryTipStr);
    outputDirectoryLabel.setToolTipText(outputDirectoryTipStr);
    outputDirectoryTextField.setToolTipText(outputDirectoryTipStr);
    outputDirectoryPanel.add(outputDirectoryTextField,  BorderLayout.CENTER);
    outputDirectoryBrowseButton.setEnabled(false);
    outputDirectoryBrowseButton.addActionListener(new ActionListener()
        {
          public void actionPerformed(ActionEvent e)
          {
            outputDirectoryBrowseButton_actionPerformed(e);
          }
        });
    outputDirectoryBrowseButton.setText("Browse");
    outputDirectoryBrowseButton.setToolTipText(
                                             "Browse for output directory");
    outputDirectoryPanel.add(outputDirectoryBrowseButton,  BorderLayout.EAST);
    panel4.add(outputDirectoryPanel,    new GridBagConstraints(0, 6, 10, 1, 1.0, 0.5
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(5, 10, 5, 10), 6, 0));

    panel5.setBorder(BorderFactory.createLoweredBevelBorder());
    panel5.setLayout(panel5Layout);
                   //lower 'weighty' param to reduce space under components:
    panel1.add(panel5, new GridBagConstraints(0, 3, 1, 1, 1.0, 0.4,
                         GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                                             new Insets(0, 0, 0, 0), 0, 0));

    plotDisplayOptionPanel.setBorder(BorderFactory.createTitledBorder("Plot Options"));
    plotDisplayOptionPanel.setToolTipText("Configure options for generated plots");
    panel5.add(plotDisplayOptionPanel, new GridBagConstraints(0, 0, 2, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));

//    plotAmpPhaseButton.setEnabled(false);
//    plotAmpPhaseButton.addActionListener(new ActionListener()
//        {
//          public void actionPerformed(ActionEvent e)
//          {
//            plotAmpPhaseButton_actionPerformed(e);
//          }
//        });
    // hide the button until it is ready
//    plotAmpPhaseButton.setVisible(false);
//    plotAmpPhaseButton.setText("Plot Amp/Phase");
//    panel5.add(plotAmpPhaseButton,  new GridBagConstraints(2, 0, 1, 1, 1.0, 0.0
//        ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 10, 0, 10), 0, 0));
//
//    panel5.add(plotSpaceEmptyLabel, new GridBagConstraints(3, 0, 1, 1, 6.0, 0.0
//        ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));

//    panel6.setBorder(BorderFactory.createLoweredBevelBorder());
//    panel6.setLayout(panel6Layout);
//    panel1.add(panel6, new GridBagConstraints(0, 4, 1, 1, 1.0, 1.0
//        ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));

    closeAllPlotsButton.addActionListener(new ActionListener()
        {
          public void actionPerformed(ActionEvent e)
          {
            closeAllPlotsButton_actionPerformed(e);
          }
        });
    closeAllPlotsButton.setForeground(Color.black);
    closeAllPlotsButton.setText("Close All Plots");
    closeAllPlotsButton.setToolTipText("Close all currently displayed plots");
//    panel5.add(closeAllPlotsButton, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0
//        ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(10, 0, 12, 0), 0, 0));
    panel5.add(closeAllPlotsButton, new GridBagConstraints(1, 1, 1, 1, 0.2, 1.0
        ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(10, 0, 12, 0), 0, 0));

    setDefaultsAllPlotsButton.addActionListener(new ActionListener()
        {
          public void actionPerformed(ActionEvent e)
          {
            setDefaultsAllPlotsButton_actionPerformed(e);
          }
        });
    setDefaultsAllPlotsButton.setForeground(Color.black);
    setDefaultsAllPlotsButton.setText("Update Options on All Plots");
    setDefaultsAllPlotsButton.setToolTipText(
                         "Update options on all currently displayed plots");
//    panel5.add(setDefaultsAllPlotsButton,  new GridBagConstraints(1, 1, 1, 1, 1.0, 1.0
//            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(10, 0, 12, 0), 0, 0));
    panel5.add(setDefaultsAllPlotsButton,  new GridBagConstraints(0, 1, 1, 1, 0.2, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(10, 0, 12, 0), 0, 0));

    panel7.setBorder(BorderFactory.createLoweredBevelBorder());
    panel7.setLayout(panel7Layout);
    panel1.add(panel7, new GridBagConstraints(0, 4, 1, 1, 1.0, 1.0
        ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));

    runEvalrespButton.addActionListener(new ActionListener()
        {
          public void actionPerformed(ActionEvent e)
          {
            runEvalrespButton_actionPerformed(e);
          }
        });
    runEvalrespButton.setForeground(Color.black);
    runEvalrespButton.setText("Run JEvalResp");
    runEvalrespButton.setToolTipText("Generate response output files");
    panel7.add(runEvalrespButton, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
        ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 1, 0));

    plotButton.addActionListener(new ActionListener()
        {
          public void actionPerformed(ActionEvent e)
          {
            plotButton_actionPerformed(e);
          }
        });
    plotButton.setForeground(Color.black);
    plotButton.setText("Plot");
    plotButton.setToolTipText("Generate response output plots");
    panel7.add(plotButton, new GridBagConstraints(1, 0, 1, 1, 1.0, 1.0
        ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 65, 0));

    if(quitButton != null)
    {
      //"Quit" button object was given; add it to panel
      panel7.add(quitButton, new GridBagConstraints(2, 0, 1, 1, 1.0, 0.0,
          GridBagConstraints.CENTER, GridBagConstraints.NONE,
          new Insets(0, 0, 0, 0), 58, 0));
    }

    aboutButton.addActionListener(new ActionListener()
        {
          public void actionPerformed(ActionEvent e)
          {
            showHelpAboutDialog();
          }
        });
    aboutButton.setForeground(Color.black);
    aboutButton.setText("About");
    aboutButton.setToolTipText("Show program information");
    panel7.add(aboutButton, new GridBagConstraints(3, 0, 1, 1, 1.0, 1.0
        ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 55, 0));

    helpButton.addActionListener(new ActionListener()
        {
          public void actionPerformed(ActionEvent e)
          {
            showFileDialog(HELP_FNAME,"JPlotResp Help");
          }
        });
    helpButton.setForeground(Color.black);
    helpButton.setText("Help");
    helpButton.setToolTipText("Show help information");
    panel7.add(helpButton, new GridBagConstraints(4, 0, 1, 1, 1.0, 1.0
        ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 59, 0));
  }

  /**
   * Sets up the file choosers objects.
   */
  private void setupFileChoosers()
  {
    //setup file chooser for "Filename":
    JFileChooser chooserObj = plotRunObj.createFileChooser(chooserFrameWidth,
        chooserFrameHeight,true);
    chooserObj.setDialogTitle("Select Response Files or Directory");
    // Create Filters
    final String respText = "resp.*";
    final IstiFileFilter respFilter =
        new IstiFileFilter(respText, "RESP Files");
    chooserObj.addChoosableFileFilter(respFilter);
    // allow multiple files to be selected
    chooserObj.setMultiSelectionEnabled(true);
    // allow directories as well as files to be selected
    chooserObj.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
    filenameChooser = chooserObj;      //set handle to chooser
    filenameBrowseButton.setEnabled(true);

    //setup file chooser for "Server":
    chooserObj = plotRunObj.createFileChooser(chooserFrameWidth,
                                   chooserFrameHeight,false);
    chooserObj.setDialogTitle("Select Server Properties File");
    // Create Filters
    final String propText = "*.prop";
    final IstiFileFilter propFilter =
        new IstiFileFilter(propText, "Prop Files");
    chooserObj.addChoosableFileFilter(propFilter);
    serverChooser = chooserObj;        //set handle to chooser
    serverBrowseButton.setEnabled(true);

    //setup file chooser for output directory
    chooserObj = plotRunObj.createFileChooser(chooserFrameWidth,
                                   chooserFrameHeight,false);
    chooserObj.setDialogTitle("Output Directory");
    chooserObj.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
    outputDirectoryChooser = chooserObj;  //set handle to chooser
    outputDirectoryBrowseButton.setEnabled(true);

/*
    //setup file chooser for "Plot Amp/Phase":
    chooserObj = plotRunObj.createFileChooser(chooserFrameWidth,
                                   chooserFrameHeight,false);
    chooserObj.setDialogTitle(plotAmpPhaseButton.getText());
    // Create Filters
    final String ampText = "amp.*";
    final String phaseText = "phase.*";
    final IstiFileFilter ampFilter =
                                 new IstiFileFilter(ampText, "AMP Files");
    final IstiFileFilter phaseFilter =
                             new IstiFileFilter(phaseText, "PHASE Files");
    final IstiFileFilter ampPhaseFilter = new IstiFileFilter(
                new String[] {ampText, phaseText}, "AMP and PHASE Files");
    final IstiFileFilter apFilter =
                             new IstiFileFilter("ap.", "Amp/Phase Files");
    chooserObj.addChoosableFileFilter(ampFilter);
    chooserObj.addChoosableFileFilter(phaseFilter);
    chooserObj.addChoosableFileFilter(ampPhaseFilter);
    chooserObj.addChoosableFileFilter(apFilter);
    // allow multiple files to be selected
    chooserObj.setMultiSelectionEnabled(true);
    plotChooser = chooserObj;          //set handle to chooser
    plotAmpPhaseButton.setEnabled(true);
*/
  }

  /**
   * Returns an array of the GUI components for which the 'Enter' key
   * should be converted to the 'Tab' key.
   * @return An array of components.
   */
  public Component [] getTabKeyCompsArray()
  {
    return new Component [] { networkTextField, stationTextField,
                      locationTextField, channelTextField, minFreqTextField,
          maxFreqTextField, numFreqsTextField, beginTimePanel.yearTextField,
            beginTimePanel.julianDayTextField, beginTimePanel.timeTextField,
                endTimePanel.yearTextField, endTimePanel.julianDayTextField,
         endTimePanel.timeTextField, startStageTextField, endStageTextField,
                    b62XValueTextField, tensionTextField, filenameTextField,
                                serverTextField, outputDirectoryTextField };
  }

  /**
   * key was pressed in the Filename Text Field
   * @param e key event
   */
  private void filenameTextField_keyPressed(KeyEvent e)
  {
    // select the filename radio button
    filenameRadioButton.setSelected(true);
  }

  /**
   * key was pressed in the Server Text Field
   * @param e key event
   */
  private void serverTextField_keyPressed(KeyEvent e)
  {
    // select the server radio button
    serverRadioButton.setSelected(true);
  }

  /**
   * Close all plots button was pressed
   * @param e key event
   */
  private void closeAllPlotsButton_actionPerformed(ActionEvent e)
  {
    closeAllPlots();
  }

  /**
   * Set defaults all plots button was pressed
   * @param e key event
   */
  private void setDefaultsAllPlotsButton_actionPerformed(ActionEvent e)
  {
    Component[] tabComponents = tabbedPaneObj.getComponents();
    for (int i = 0; i < tabComponents.length; i++)
    {
      Component tabComponent = tabComponents[i];
      if (tabComponent instanceof RespPlotPanel)
      {
        //set defaults
        ((RespPlotPanel)tabComponent).setPlotDisplayOptions(
            plotDisplayOptionPanel);
      }
    }
  }

  /**
   * "Run JEvalresp" button was pressed
   * @param e key event
   */
  private void runEvalrespButton_actionPerformed(ActionEvent e)
  {
    processAndOutput(false); // processing without plot
  }

  /**
   * Plot button was pressed
   * @param e key event
   */
  private void plotButton_actionPerformed(ActionEvent e)
  {
    processAndOutput(true); // processing with plot
  }

  /**
   * Displays the "About" dialog window.
   */
  private void showHelpAboutDialog()
  {
    RespAboutPanel panel = new RespAboutPanel(launchBrowserObj);
    SimpleDialog dlg = new SimpleDialog(this, "About JPlotResp", true,
                                        panel, new JButton("Close"));
    dlg.setResizable(false);
    dlg.setVisible(true);
  }

  /** Add the items to the units combo box */
  private void unitsComboBoxAddItems()
  {
    // add the items for the units combo box from JEvalResp
    for (int i = 0; i < Run.UNIT_CONV_LONGSTRS.length; i++)
    {
      unitsComboBox.addItem(Run.UNIT_CONV_LONGSTRS[i]);
    }
  }

  /** Add the items to the frequency spacing combo box */
  private void frequencySpacingComboBoxAddItems()
  {
    // add the items for the frequency spacing combo box from JEvalResp
    for (int i = 0; i < Run.TYPE_SPACE_LONGSTRS.length; i++)
    {

      frequencySpacingComboBox.addItem(Run.TYPE_SPACE_LONGSTRS[i]);
    }
  }

  /** Add the items to the response type combo box */
  private void responseTypeComboBoxAddItems()
  {
    // add the items for the response type combo box from JEvalResp
    for (int i = 0; i < Run.RESP_TYPE_LONGSTRS.length; i++)
    {
      responseTypeComboBox.addItem(Run.RESP_TYPE_LONGSTRS[i]);
    }
  }

  /**
   * Changes the buttons that are enabled/disabled while processing.
   * @param processFlag a boolean where true indicates process is running.
   */
  private void changeProcessingButtons(boolean processFlag)
  {
    boolean buttonsDisabledWhileProcessing = processFlag?false:true;

    // buttons that are disabled while processing
    if(consolePanel != null)
      consolePanel.closeButton.setEnabled(buttonsDisabledWhileProcessing);
    plotButton.setEnabled(buttonsDisabledWhileProcessing);
    runEvalrespButton.setEnabled(buttonsDisabledWhileProcessing);
  }

  /**
   * Display the user supplied parameters.
   * @param plotFlag a boolean where true indicates output should be plotted.
   */
  private void displayParameters(boolean plotFlag)
  {
    final PrintStream out = getConsoleOut();
    if(consolePanel != null && consolePanel.getContainsTextFlag())
    {
      //console contains text
      out.println();
      out.println(         //show separator
                         "--------------------------------------------------");
    }
    out.println("Generating " + (plotFlag?"plots":"files") +
                       " with the following parameters:");
    out.print(
        "\"" + stationTextField.getText() + "\", " +
        "\"" + channelTextField.getText() + "\", " +
        "\"" + networkTextField.getText() + "\", " +
        "\"" + locationTextField.getText() + "\", " +
        "" + minFreqTextField.getText() + ", " +
        "" + maxFreqTextField.getText() + ", " +
        "" + numFreqsTextField.getText() + ", " +
        beginTimePanel.getText() + ", " +
        endTimePanel.getText());
    if (serverRadioButton.isSelected())
      out.print(" (from server)");
    out.println("\n--------------------------------------------------");
  }

  /**
   * Process parameters and generate output.
   * @param plotFlag a boolean where true indicates output should be plotted.
   */
  protected void processAndOutput(boolean plotFlag)
  {
    if(consolePanel == null)      //if not yet created then create console
    {
      consolePanel = new RespConsolePanel(
          tabbedPaneObj, plotRunObj.isSetupStreams());
    }
    // add the console panel
    consolePanel.addTabbedPane();

    // display the parameters that will be processed
    displayParameters(plotFlag);
    // change the buttons that change while processing
    changeProcessingButtons(true);

    // create and start a new thread for processing
    new RespProcessingThread(plotRunObj, this, plotFlag);
  }

  /**
   * Cleanup that is done after processing is completed.
   * @param returnedFlag return flag from JEvalResp.
   */
  protected void processAndOutputCompleted(boolean returnedFlag)
  {
         //restore the buttons that changed while processing:
    changeProcessingButtons(false);
         //if error message generated then display it:
    if(!returnedFlag && plotRunObj.getErrorFlag())
    {
      getConsoleOut().println(plotRunObj.getErrorMessage());
    }
  }

  /**
   * Requests that the "Network:" text entry field have the keyboard
   * focus.
   */
  public void setInitialFocus()
  {
    networkTextField.requestFocus();
  }

  /**
   * Perform cleanup operations before program termination.
   */
  protected void exitCleanup()
  {
    //if saving the configuration
    if (rememberSettingsCheckBox.isSelected())
    {
      cm.saveComponentConfigSettings();  //save the component settings
      plotRunObj.saveConfig();  //save the configuration
    }
    //if configuration has remember-settings selected or frame moved/resized
    else if (plotRunObj.cfgObj.rememberSettingsProp.booleanValue() ||
                                                plotRunObj.isFrameChanged())
    {                   //first, set all items to their default values:
      plotRunObj.cfgObj.setAllItemsToDefaults();
                        //then, set the remember-settings flag to false:
      plotRunObj.cfgObj.rememberSettingsProp.setValue(false);
      plotRunObj.saveConfig();  //save the configuration
    }
    if(consolePanel != null)
    {
      consolePanel.close();
      consolePanel = null;
    }
  }

  /**
   * Close all plots.
   */
  protected void closeAllPlots()
  {
    Component[] tabComponents = tabbedPaneObj.getComponents();
    for (int i = 0; i < tabComponents.length; i++)
    {
      Component tabComponent = tabComponents[i];
      if (tabComponent instanceof RespPlotPanel)
      {
        tabbedPaneObj.remove(tabComponent);
      }
    }
  }

  /**
   * Gets the current selection.
   * @param fc file chooser.
   * @return the current selection.
   */
  public static String getCurrentSelection(JFileChooser fc)
  {
    if (fc instanceof IstiFileChooser)
      return ((IstiFileChooser)fc).getCurrentSelection();
    else
      return fc.getSelectedFile().getPath();
  }

  /**
   * Sets the current selection to be used when the dialog is shown
   * @param fc file chooser.
   * @param currentFile current file selection
   */
  public static void setCurrentSelection(JFileChooser fc, String currentFile)
  {
    if (fc instanceof IstiFileChooser)
      ((IstiFileChooser)fc).setCurrentSelection(currentFile);
    else
      fc.setSelectedFile(new File(currentFile));
  }

  /**
   * Filename "Browse" button was pressed
   * @param e key event
   */
  private void filenameBrowseButton_actionPerformed(ActionEvent e)
  {
    if(filenameChooser != null)
    {
      setCurrentSelection(filenameChooser, filenameTextField.getText());
      if (filenameChooser.showDialog(this,
                                     IstiFileChooser.APPROVE_BUTTON_TEXT) ==
                                     JFileChooser.APPROVE_OPTION)
      {
        // select the filename radio button
        filenameRadioButton.setSelected(true);
        filenameTextField.setText(getCurrentSelection(filenameChooser));
      }
    }
    // set the focus on the associated text field
    filenameTextField.requestFocus();
  }

  /**
   * Server "Browse" button was pressed
   * @param e key event
   */
  private void serverBrowseButton_actionPerformed(ActionEvent e)
  {
    if(serverChooser != null)
    {
      setCurrentSelection(serverChooser, serverTextField.getText());
      if (serverChooser.showDialog(this, IstiFileChooser.APPROVE_BUTTON_TEXT)
          == JFileChooser.APPROVE_OPTION)
      {
        // select the server radio button
        serverRadioButton.setSelected(true);
        serverTextField.setText(getCurrentSelection(serverChooser));
      }
    }
    // set the focus on the associated text field
    serverTextField.requestFocus();
  }

  /**
   * Output directory browse button was pressed
   * @param e key event
   */
  void outputDirectoryBrowseButton_actionPerformed(ActionEvent e)
  {
    if (outputDirectoryChooser != null)
    {
      final String outputDirectoryText =
                                  outputDirectoryTextField.getText().trim();
      final File outputDirectory;

      if (outputDirectoryText.length() <= 0 ||
                                   outputDirectoryText.equals("."))
      {  //output directory is empty or "."
              //use current directory, expanded out to full pathname
        outputDirectory = new File(".").getAbsoluteFile();
      }
      else
        outputDirectory = new File(outputDirectoryText);

      outputDirectoryChooser.setSelectedFile(outputDirectory);
      if (outputDirectoryChooser.showDialog(this,
          IstiFileChooser.APPROVE_BUTTON_TEXT) == JFileChooser.APPROVE_OPTION)
      {
        String outDirStr =
                         outputDirectoryChooser.getSelectedFile().getPath();
        final int len;
            //if ends with "." then remove it
        if((len=outDirStr.length()) > 1 && outDirStr.endsWith("."))
          outDirStr = outDirStr.substring(0,len-1);
        outputDirectoryTextField.setText(outDirStr);
      }
    }

    // set the focus on the associated text field
    outputDirectoryTextField.requestFocus();
  }

  /**
   * Displays the specified HTML file in a dialog window.
   * @param nameStr the file name string or menu name string.
   * @param titleStr the title to use.
   */
   public void showFileDialog(String nameStr,String titleStr)
   {
     viewHTMLPanelHandler.showFileDialog(nameStr,titleStr,
                                         plotRunObj.programFrameObj,null,true);
   }

  /**
   * "Plot Amp/Phase" button was pressed
   * @param e key event
   */
//  void plotAmpPhaseButton_actionPerformed(ActionEvent e)
//  {
//    if(plotChooser != null)
//    {
//      if(plotChooser.showDialog(this, IstiFileChooser.APPROVE_BUTTON_TEXT)
//         == JFileChooser.APPROVE_OPTION)
//      {
//        // PRINT OUT DATA FOR TESTING
//        getPrintStream().println("Plotting data from: " +
//                           getCurrentSelection(plotChooser));
//      }
//    }
//  }
}
