//RespTimePanel.java:  Implements the time panel.
//
//  7/12/2002 -- [KF]
//  7/17/2002 -- [ET]  Changed 'GridBagLayout' to 'BoxLayout' with
//                     sub-panels to better handle resizing.
//   8/6/2002 -- [ET]  Changed references to static 'FilteredJTextField'
//                     strings from local to new 'FilteredJTextField'
//                     versions ('TOD_CHARS', etc); added tool-tip
//                     parameter to constructor; changed access on text
//                     edit fields from 'protected' to "package".
//  5/14/2003 -- [ET]  Made all label and text-field objects be declared
//                     "public final".
//  4/11/2005 -- [ET]  Improved imports.
//

package com.isti.jplotresp;

import java.awt.Color;
import java.awt.FlowLayout;
import javax.swing.*;
import javax.swing.border.TitledBorder;
import com.isti.util.gui.FilteredJTextField;

/**
 * Class RespTimePanel implements the time panel.
 */
public class RespTimePanel extends JPanel
{
  protected final BoxLayout layoutObj = new BoxLayout(this,BoxLayout.X_AXIS);
  protected final TitledBorder borderObj =
      new TitledBorder(BorderFactory.createEtchedBorder(), "Time");
  public final JLabel yearLabel = new JLabel();
  public final FilteredJTextField yearTextField = new FilteredJTextField("", 4,
      FilteredJTextField.INTEGER_CHARS + FilteredJTextField.WILDCARD_ALL_CHAR);
  public final JLabel julianDayLabel = new JLabel();
  public final FilteredJTextField julianDayTextField =
      new FilteredJTextField("", 3,
      FilteredJTextField.INTEGER_CHARS + FilteredJTextField.WILDCARD_ALL_CHAR);
  public final JLabel timeLabel = new JLabel();
  public final FilteredJTextField timeTextField = new FilteredJTextField("",
      8, FilteredJTextField.TOD_CHARS);

  /**
   * Creates a time panel with the default title "Time".
   */
  public RespTimePanel()
  {
    this(null,null);
  }

  /**
   * Creates a time panel with the given title and tool-tip string.
   * @param titleStr title string
   * @param toolTipStr tool tip string
   */
  public RespTimePanel(String titleStr,String toolTipStr)
  {
    borderObj.setTitleColor(Color.black);
    if (titleStr != null)
      borderObj.setTitle(titleStr);
    if(toolTipStr != null)
      setToolTipText(toolTipStr);
    try
    {
      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /**
   * Component initialization
   * @throws Exception
   */
  private void jbInit() throws Exception
  {
    setBorder(BorderFactory.createCompoundBorder(borderObj,
        BorderFactory.createEmptyBorder(4,0,0,0)));
    setLayout(layoutObj);

    yearLabel.setForeground(Color.black);
    yearLabel.setHorizontalAlignment(SwingConstants.CENTER);
    yearLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    yearLabel.setText("Year: ");
    final String yearTextTip = "Year to be matched";
    yearLabel.setToolTipText(yearTextTip);

    yearTextField.setToolTipText(yearTextTip);

    final JPanel yearPanel = new JPanel(new FlowLayout());
    yearPanel.add(yearLabel);
    yearPanel.add(yearTextField);
    add(yearPanel);

    julianDayLabel.setForeground(Color.black);
    julianDayLabel.setHorizontalAlignment(SwingConstants.CENTER);
    julianDayLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    julianDayLabel.setText("Julian Day: ");
    final String julianDayTextTip = "Julian day-of-the-year to be matched";
    julianDayLabel.setToolTipText(julianDayTextTip);

    julianDayTextField.setToolTipText(julianDayTextTip);

    final JPanel jDayPanel = new JPanel(new FlowLayout());
    jDayPanel.add(julianDayLabel);
    jDayPanel.add(julianDayTextField);
    add(jDayPanel);

    timeLabel.setForeground(Color.black);
    timeLabel.setHorizontalAlignment(SwingConstants.CENTER);
    timeLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    timeLabel.setText("Time: ");
    final String timeTipText = "Time-of-day value (HH:MM:SS)";
    timeLabel.setToolTipText(timeTipText);

    timeTextField.setToolTipText(timeTipText);

    final JPanel timePanel = new JPanel(new FlowLayout());
    timePanel.add(timeLabel);
    timePanel.add(timeTextField);
    add(timePanel);
  }

  /**
   * @return text for displaying the user supplied parameters.
   */
  public String getText()
  {
    String string = new String("(" + yearTextField.getText() + ", " +
                               "" + julianDayTextField.getText());
    if (timeTextField.getText().length() > 0)   //only show time if specified
      string += new String(", " + "\"" + timeTextField.getText() + "\"");
    string += ")";
    return string;
  }
}
